import os.path
#
# https://googleapis.github.io/google-api-python-client/docs/dyn/tasks_v1.tasks.html
#
from google.auth.transport.requests import Request
from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow
from googleapiclient.discovery import build
from googleapiclient.errors import HttpError

from console import fg, bg
import datetime as dt
import click
import sys

# If modifying these scopes, delete the file token.json.
#SCOPES = ["https://www.googleapis.com/auth/tasks.readonly"]
SCOPES = ["https://www.googleapis.com/auth/tasks"]
# --- token will happen on the PC if it doesnt yet exist
TOKEN = "~/.google_tasks_api.token"
TOKEN = os.path.expanduser(TOKEN)

# --- this is credential given by google
CRED = "~/.credentials_google0.json"
CRED =    os.path.expanduser(CRED)


def to_rfc3339(date_str):
    # Convert date like '2025-07-12 Sat 00:00' or ISO to RFC3339
    try:
        dt_obj = dt.datetime.strptime(date_str, "%Y-%m-%dT%H:%M:%S.%fZ")
    except ValueError:
        dt_obj = dt.datetime.strptime(date_str, "%Y-%m-%d %a %H:%M")
    return dt_obj.strftime("%Y-%m-%dT%H:%M:%S.000Z")


def dt_to_org(datim):
    """
    Convert date string to org-mode format.
    Supports ISO format with 'T' and 'Z' or 'YYYY-MM-DD Ddd HH:MM'.
    """
    try:
        due_date = dt.datetime.strptime(datim, "%Y-%m-%dT%H:%M:%S.%fZ").date()
    except ValueError:
        try:
            due_date = dt.datetime.strptime(datim, "%Y-%m-%d %a %H:%M").date()
        except ValueError:
            # fallback: return input as is
            return datim
    return due_date.strftime("%Y-%m-%d %a %H:%M")

# def dt_to_org(datim):
#     """
#     from T Z to <2025-06-30 Mon 00:00>
#     """
#     due_str = datim # item['due']    # e.g. '2025-07-01T00:00:00.000Z'
#     due_date = dt.datetime.strptime(due_str, "%Y-%m-%dT%H:%M:%S.%fZ").date()  # compare to today etc..
#     p_date =  due_date.strftime("%Y-%m-%d")
#     p_time =  due_date.strftime("%H:%M:%S")
#     p_wday =  due_date.strftime("%F")
#     #
#     result = due_date.strftime("%Y-%m-%d %a %H:%M")
#     return result
#     #
#     due_str = due_str.split("T")[0]
#     # plain date
#     today = dt.datetime.now(dt.timezone.utc).date()
#     tomorrow = today + dt.timedelta(days=1)


# ================================================================================
#
# --------------------------------------------------------------------------------

def tasks_read_remote(service, TASKLIST1):
    """

    """
    tasks_remote = {}
    for k, v in TASKLIST1.items():
        tasks_remote[v] = []
        try:
            results = service.tasks().list(tasklist=k).execute()
            items = results.get("items", [])
            for item in items:
                if item.get('status') == "completed":
                    continue
                task = {
                    "title": item.get("title"),
                    "due": item.get("due"),
                    "updated": item.get("updated"),
                    "status": item.get("status", "needsAction"),
                    "notes": item.get("notes", ""),
                }
                tasks_remote[v].append(task)
        except HttpError as err:
            print("X...", err)
    return tasks_remote
    # TASKS_REMOTE = []
    # #print("D... ============================== Parsing tasklists ===")
    # TASKS_REMOTE.append("* GOOGLE TASKS")
    # for k, v in TASKLIST1.items():
    #     TASKS_REMOTE.append(f"** {v}" )
    #     try:
    #         #service = build("tasks", "v1", credentials=creds)
    #         # Call the Tasks API
    #         results = service.tasks().list(tasklist=k).execute()
    #         items = results.get("items", [])

    #         #print()
    #         for item in items:

    #             if 'status' in item:
    #                 if item['status'] == "completed":
    #                     #print(f"{fg.slategray}     - {item['title']} : STAT: {item['status']}    {fg.default}")
    #                     continue

    #             TASKS_REMOTE.append(f"*** TODO {item['title']}")
    #             if 'due' in item:
    #                 s_scheduled = dt_to_org(item['due'])
    #                 TASKS_REMOTE.append(f"SCHEDULED: <{s_scheduled}>")
    #             s_updated = dt_to_org(item['updated'])
    #             TASKS_REMOTE.append(f"UPDATED: [{s_updated}]") # inactive timestamp
    #             if 'status' in item:
    #                 if item['status'] != "needsAction":
    #                     TASKS_REMOTE.append(f"            - STAT: {item['status']}    ")
    #             if 'due' in item:
    #                 s_scheduled = dt_to_org(item['due'])
    #                 due_str = item['due']    # e.g. '2025-07-01T00:00:00.000Z'
    #                 due_date = dt.datetime.strptime(due_str, "%Y-%m-%dT%H:%M:%S.%fZ").date()
    #                 #due_str = due_str.split("T")[0]
    #                 today = dt.datetime.now(dt.timezone.utc).date()
    #                 tomorrow = today + dt.timedelta(days=1)
    #                 if due_date == today:
    #                     TASKS_REMOTE.append(f"             -  Due today     ") # The allow day only via API
    #                 elif due_date == tomorrow:
    #                     TASKS_REMOTE.append(f"             -  Due tomorrow  ") # They let day only
    #                 elif due_date < today:
    #                     TASKS_REMOTE.append(f"             -  Due in the PAST    {due_str}") # They let day only
    #                 else:
    #                     #print("Due on", due_date)
    #                     TASKS_REMOTE.append(f"             - DUE : {due_str}   ")
    #             if 'notes' in item:
    #                 TASKS_REMOTE.append(f"             - NOTES: {item['notes']}    ")
    #             #results = service.tasks().get(tasklist=k, task=item['id'])
    #     except HttpError as err:
    #         print("X...", err)
    # return TASKS_REMOTE

def dict2org(tasks_dict):
    """
    Converts nested dict {tasklist: [tasks]} to org-mode lines.
    """
    lines = ["* GOOGLE TASKS"]
    for tasklist, tasks in tasks_dict.items():
        lines.append(f"** {tasklist}")
        for t in tasks:
            lines.append(f"*** TODO {t['title']}")
            if t.get("due"):
                due_org = dt_to_org(t["due"])
                lines.append(f"SCHEDULED: <{due_org}>")
            if t.get("updated"):
                #updated_org = dt_to_org(t["updated"])
                #lines.append(f"UPDATED: [{updated_org}]")
                lines.append(f"UPDATED: [{t['updated']}]")  # store full ISO timestamp
            if t.get("status") and t["status"] != "needsAction":
                lines.append(f"            - STAT: {t['status']}")
            if t.get("notes"):
                lines.append(f"            - NOTES: {t['notes']}")
    return lines

# ================================================================================
#
# --------------------------------------------------------------------------------

def org2dict(org_lines):
    tasks_dict = {}
    current_tasklist = None
    current_task = None
    for line in org_lines:
        line = line.strip()
        if line.startswith("* GOOGLE TASKS"):
            continue
        elif line.startswith("** "):
            # Save previous task before switching tasklist
            if current_task and current_tasklist:
                tasks_dict[current_tasklist].append(current_task)
                current_task = None
            current_tasklist = line[3:]
            tasks_dict.setdefault(current_tasklist, [])
        elif line.startswith("*** TODO "):
            # Save previous task before starting new one
            if current_task and current_tasklist:
                tasks_dict[current_tasklist].append(current_task)
            current_task = {"title": line[8:].strip(), "due": None, "updated": None, "status": "needsAction", "notes": ""}
        elif line.startswith("SCHEDULED: <") and current_task:
            current_task["due"] = line[len("SCHEDULED: <"):-1]
        elif line.startswith("UPDATED: [") and current_task:
            current_task["updated"] = line[len("UPDATED: ["):-1]
        elif line.strip().startswith("- STAT:") and current_task:
            current_task["status"] = line.split(":")[1].strip()
        elif line.strip().startswith("- NOTES:") and current_task:
            current_task["notes"] = line.split(":")[1].strip()
    # Add last task if any
    if current_task and current_tasklist:
        tasks_dict[current_tasklist].append(current_task)
    return tasks_dict
    # """
    # Parses org-mode lines into nested dict {tasklist: [tasks]}.
    # """
    # tasks_dict = {}
    # current_tasklist = None
    # current_task = None
    # for line in org_lines:
    #     line = line.strip()
    #     if line.startswith("* GOOGLE TASKS"):
    #         continue
    #     elif line.startswith("** "):
    #         current_tasklist = line[3:]
    #         tasks_dict[current_tasklist] = []
    #     elif line.startswith("*** TODO "):
    #         if current_task:
    #             tasks_dict[current_tasklist].append(current_task)
    #         current_task = {"title": line[8:], "due": None, "updated": None, "status": "needsAction", "notes": ""}
    #     elif line.startswith("SCHEDULED: <") and current_task:
    #         current_task["due"] = line[len("SCHEDULED: <"):-1]
    #     elif line.startswith("UPDATED: [") and current_task:
    #         current_task["updated"] = line[len("UPDATED: ["):-1]  # read full ISO timestamp
    #         # current_task["updated"] = line[len("UPDATED: ["):-1]
    #     elif line.strip().startswith("- STAT:") and current_task:
    #         current_task["status"] = line.split(":")[1].strip()
    #     elif line.strip().startswith("- NOTES:") and current_task:
    #         current_task["notes"] = line.split(":")[1].strip()
    # if current_task and current_tasklist:
    #     tasks_dict[current_tasklist].append(current_task)
    # return tasks_dict

def tasks_read_file(filepath="tasks_local.org"):
    with open(filepath, "r") as f:
        lines = f.readlines()
    return org2dict(lines)


# def tasks_read_file(filepath="tasks_local.org"):
#     """
#     Reads local tasks from an org-mode formatted file.
#     Returns a list of task dicts with keys: title, due, updated, status, notes.
#     """
#     if not os.path.exists(filepath):
#         print(f"X... NO FILE {filepath}")
#         sys.exit(0)
#     tasks = []
#     current_task = None
#     with open(filepath, "r") as f:
#         for line in f:
#             line = line.strip()
#             if line.startswith("*** TODO "):
#                 if current_task:
#                     tasks.append(current_task)
#                 current_task = {"title": line[8:], "due": None, "updated": None, "status": "needsAction", "notes": None}
#             elif line.startswith("SCHEDULED: <") and current_task:
#                 # parse date inside <>
#                 current_task["due"] = line[len("SCHEDULED: <"):-1]
#             elif line.startswith("UPDATED: [") and current_task:
#                 current_task["updated"] = line[len("UPDATED: ["):-1]
#             elif line.strip().startswith("- STAT:") and current_task:
#                 current_task["status"] = line.split(":")[1].strip()
#             elif line.strip().startswith("- NOTES:") and current_task:
#                 current_task["notes"] = line.split(":")[1].strip()
#         if current_task:
#             tasks.append(current_task)
#     return tasks


# ================================================================================
#
# --------------------------------------------------------------------------------

def tasks_compare_rl(remote_dict, local_dict):
    """
    Compare remote and local nested dicts
    """
    # drop_remote = {} # NO DROP REMOTE
    to_update = []
    to_push_new = []

    for tasklist, remote_tasks in remote_dict.items():
        if tasklist in local_dict:
            local_tasks = local_dict[tasklist]
            for ri in remote_tasks:
                present = False
                needsupdate = True
                #print(f"/{ri['title']}/")
                for li in local_tasks:
                    #print(f"         /{li['title']}/")
                    if (ri['title'] == li['title']):
                        present = True
                        if ri['updated'] == li['updated']:
                        #print(ri['title'], "ok")
                            needsupdate = False
                if not present:
                    print(f"r2l... @ {tasklist:20s}   #   {ri['title']}")
                elif needsupdate:
                    print(f"r2l... @ {tasklist:20s}   U   {ri['title']}")
        else:
            print("X... no tasklist {tasklist} in local")
    print("_" * 60)

    for tasklist, local_tasks in local_dict.items():
        if tasklist in remote_dict:
            remote_tasks = remote_dict[tasklist]
            for li in local_tasks:
                present = False
                needsupdate = True
                #print(f"/{ri['title']}/")
                for ri in remote_tasks:
                    #print(f"         /{li['title']}/")
                    if (ri['title'] == li['title']):
                        present = True
                        if ri['updated'] == li['updated']:
                            needsupdate = False
                        #print(ri['title'], "ok")
                if not present:
                    print(f"l2r... @ {tasklist:20s}   #   {li['title']}")
                    to_push_new.append( (tasklist, li['title']) )
                elif needsupdate:
                    print(f"l2r... @ {tasklist:20s}   U   {li['title']}")
                    to_update.append( (tasklist, li['title']) )
        else:
            print("X... no tasklist {tasklist} in remote")


    return to_push_new, to_update



# ================================================================================
#
# --------------------------------------------------------------------------------

def add_tasks_to_remote(service, TASKLIST1, local_dict, tasks_to_add):
    """
    Adds local tasks specified by (tasklist, title) tuples to remote Google Tasks.
    service: Google Tasks API service object.
    TASKLIST1: dict mapping tasklist IDs to titles.
    local_dict: nested dict {tasklist_title: [tasks]}.
    tasks_to_add: list of (tasklist_title, task_title) tuples.
    """
    # Reverse TASKLIST1 to map title -> id
    title_to_id = {v: k for k, v in TASKLIST1.items()}

    for tasklist_title, task_title in tasks_to_add:
        tasklist_id = title_to_id.get(tasklist_title)
        if not tasklist_id:
            print(f"Tasklist '{tasklist_title}' not found in remote.")
            continue
        local_tasks = local_dict.get(tasklist_title, [])
        task = next((t for t in local_tasks if t["title"] == task_title), None)
        if not task:
            print(f"Task '{task_title}' not found locally in '{tasklist_title}'.")
            continue

        body = {
            "title": task["title"],
            "status": task.get("status", "needsAction"),
        }
        if task.get("due"):
            #body["due"] = task["due"]
            body["due"] = to_rfc3339(task["due"])
        if task.get("notes"):
            body["notes"] = task["notes"]

        print(f"D... task '{task_title}' to remote tasklist '{tasklist_title}'.")
        input("Add the task? > ")
        try:
            service.tasks().insert(tasklist=tasklist_id, body=body).execute()
            print(f"Added task '{task_title}' to remote tasklist '{tasklist_title}'.")
        except HttpError as err:
            print(f"Failed to add task '{task_title}': {err}")




# ================================================================================
#
# --------------------------------------------------------------------------------

@click.command()
@click.option("--command", "-c", default="rprint", help="rprint  lprint  rlcomp")
def main(command):
    """Shows basic usage of the Tasks API.
    Prints the title and ID of the first 10 task lists.
    """
    creds = None
    # The file token.json stores the user's access and refresh tokens, and is
    # created automatically when the authorization flow completes for the first
    # time.
    if os.path.exists(TOKEN):
        creds = Credentials.from_authorized_user_file(TOKEN, SCOPES)
    # If there are no (valid) credentials available, let the user log in.
    if not creds or not creds.valid:
        print(f"D...  TOKEN: {TOKEN}")
        if creds and creds.expired and creds.refresh_token:
            creds.refresh(Request())
        else:
            flow = InstalledAppFlow.from_client_secrets_file(
                    CRED, SCOPES
            )
            creds = flow.run_local_server(port=0)
        # Save the credentials for the next run
        with open(TOKEN, "w") as token:
            token.write(creds.to_json())

    TASKLIST = {}
    try:
        service = build("tasks", "v1", credentials=creds)

        # Call the Tasks API
        results = service.tasklists().list(maxResults=10).execute()
        items = results.get("items", [])

        if not items:
            print("No task lists found.")
            return

        #print("D... ===============================Task lists:")
        for item in items:
            #print(f"D...   * {item['title']} ({item['id']})")
            TASKLIST[item['id']] = item['title']
    except HttpError as err:
        print(err)

    # ===========================================================   command section ============
    if command == "rprint" or command == "r": # I NEED THIS TO CREATE ORGMODE AGENDA
        tarem = tasks_read_remote(service, TASKLIST)
        #print(tarem)
        print()
        for i in dict2org(tarem): print(i)
    # elif command == "lprint" or command == "l": # I dont need it really
    #     taloc = tasks_read_file()
    #     #print(taloc)
    #     print()
    #     for i in dict2org(taloc): print(i)
    # elif command == "rlcomp" or command == "rl":
    #     tarem = tasks_read_remote(service, TASKLIST)
    #     taloc = tasks_read_file()
    #     to_push_new, to_update = tasks_compare_rl(tarem, taloc)
    #     print("TO PUSH NEW")
    #     print(to_push_new)
    #     add_tasks_to_remote(service, TASKLIST, taloc, to_push_new)
    #     #
    #     print("TO UPDATE")
    #     print(to_update)
    #     #print("CHANGE")
    #     #print(res['to_update'])
    # else:
    #     print("X... unknown command ")

if __name__ == "__main__":
    main()



import curses
import click
import logging
import random

import jusfltuls.mcinfl_library as maninf
#from  jusfltuls.mcinfl_library import user_glob # set this for all session
import sys
import datetime as dt
import os
import time
from console import fg, bg, fx
import socket
from jusfltuls.check_new_version import is_there_new_version

"""
1.    -portable   -skip-errors   sometimes not possibnle

~/.visidatarc
# ------------------------------------------------------------
# RUNWITH  uvx --with=numpy visidata -p export_local_flashcam5000_20250522_0838_01.vdj
options.color_key_col=''
options.color_selected_row='yellow'
options.color_note_type='yellow'
#options.color_graph_hidden='blue'
options.color_column_sep='blue'
# COMMAND FOR '1'
BaseSheet.addCommand('1', 'hello-world1', 'status("Hello world! C-D saves and -p saved.vdj replays; z| sel ...; gt; gd; ")')
import numpy as np
import datetime as dt
BaseSheet.addCommand('2', 'hello-world2', 'status(f"Random sample: {np.random.sample()}  {dt.datetime.now().strftime(\'%t\')}")')
def t2d(time):
    return dt.datetime.fromtimestamp(time/1000000000)




"""

def get_left_panel_status():
    res = maninf.show_databases()
    logging.debug(f"{res}")
    return res#["file1.txt", "file2.txt", "dir1/", "file3.log"]

def get_right_panel_status():
    res = maninf.show_databases()
    logging.debug(f"{res}")# return ["doc1.pdf", "image.png", "music.mp3", "video.mp4"]
    return res
    #return ["doc1.pdf", "image.png", "music.mp3", "video.mp4"]


def show_message_box(stdscr, message, wait_key=True):
    if message is None:
        return
    lines = [line.strip() for line in message.strip().split('\n')]
    h, w = stdscr.getmaxyx()
    #
    box_h = 8
    if len(lines) > 6 and len(lines) < h - 3:
        box_h = len(lines) + 2
    elif len(lines) > 6:
        box_h = h - 1
    box_w = max(len(line) for line in lines) + 4
    if box_w > w - 2:
        box_w = w - 2
    #
    win = curses.newwin(box_h, box_w, (h - box_h)//2, (w - box_w)//2)
    win.box()
    for i, line in enumerate(lines[:box_h - 2]):
        win.addstr(1 + i, 2, line[:box_w - 4])
    win.refresh()
    if wait_key:
        win.getch()  # wait for key press
    win.clear()
    stdscr.refresh()



def confirm_dialog(stdscr, prompt):
    h, w = stdscr.getmaxyx()
    win = curses.newwin(5, len(prompt) + 12, h//2 - 2, (w - len(prompt) - 12)//2)
    win.box()
    win.addstr(1, 2, prompt)
    win.addstr(3, 4, "Yes (y) / No (n)")
    win.refresh()
    while True:
        c = win.getch()
        if c in (ord('y'), ord('Y')):
            return True
        elif c in (ord('n'), ord('N')):
            return False


def on_backup(stdscr):
    """
    """
    NOW = dt.datetime.now().strftime("%Y%m%d_%H%M")
    PATH = f"~/INFL_BACKUP_PORTABLE_{NOW}"
    #newname = input_dialog(stdscr, f"Backup Full Influx? -portable -skip-errors")
    confirm = confirm_dialog(stdscr, f"Backup Full Influx? -portable -skip-errors; {PATH}")
    if confirm:
        PATH = os.path.expanduser(PATH)
        if os.path.exists(PATH):
            pass
        else:
            os.mkdir(PATH)
        logging.debug(f"Started  backup to {PATH}")
        maninf.backup_portable(PATH)
        logging.debug(f"Done backup to {PATH}")

def on_copy(name, dbase, other_dbase, stdscr, wordcopy="COPY", asking=True):
    logging.debug(f"copying - {name}/{dbase} -> {other_dbase};")
    if stdscr:
        confirm = True
        if asking:
            confirm = confirm_dialog(stdscr, f"{wordcopy} measurement '{name}' of {dbase} TO {other_dbase}?")
        if confirm:
            if (name is None) or (dbase is None) or (other_dbase is None):
                pass
            else:
                res = maninf.copy_measurement(name, dbase, other_dbase, silent=True)
                logging.debug( f"{res}")
            logging.debug(f"copied '{name}'")
        else:
            logging.debug(f"copying cancelled for '{name}'")
    else:
        # If no stdscr provided, assume yes or handle differently
        # maninf.drop_database(name)
        logging.debug(f"copied '{name}' without confirmation")
    pass

def on_move():
    logging.debug(f"moving - ;")
    pass

def on_create_database(name):
    logging.debug(f"creating new thing - {name};")
    if name is None or len(name) == 0:
        return
    maninf.create_database(name)
    pass

def on_delete_database(name, stdscr):
    logging.debug(f"deleting database - {name};")
    if stdscr:
        confirm = confirm_dialog(stdscr, f"Delete database '{name}'?")
        if confirm:
            maninf.drop_database(name)
            logging.debug(f"Deleted '{name}'")
        else:
            logging.debug(f"Deletion cancelled for '{name}'")
    else:
        # If no stdscr provided, assume yes or handle differently
        # maninf.drop_database(name)
        logging.debug(f"Deleted '{name}' without confirmation")


def on_delete_measurement(name, database, stdscr, asking=True):
    """
    I took the same as delete database
    """
    logging.debug(f"deleting measurement - {name} from {database};")
    if stdscr:
        confirm = True
        if asking:
            confirm = confirm_dialog(stdscr, f"Delete measurement '{name}' of {database}?")
        if confirm:
            maninf.delete_measurement(name, database)
            logging.debug(f"Deleted measurement '{name}' from {database}")
        else:
            logging.debug(f"Deletion cancelled for meas. '{name}' of {database}")
    else:
        # If no stdscr provided, assume yes or handle differently
        # maninf.drop_database(name)
        logging.debug(f"Deleted '{name}' without confirmation")


# def draw_panel(win, items, selected_idx, active):
#     h, w = win.getmaxyx()
#     win.clear()
#     for idx, item in enumerate(items):
#         if idx == selected_idx:
#             mode = curses.A_REVERSE if active else curses.A_BOLD
#         else:
#             mode = curses.A_NORMAL
#         if idx < h - 2:
#             win.addstr(idx + 1, 1, item[:w-3], mode)
#     win.box()
#     win.refresh()

def draw_panel(win, items, selected_idx, active, offset):
    """
    updated for offset to enable scrolling
    """
    h, w = win.getmaxyx()
    win.clear()
    visible_items = items[offset:offset + h - 2]
    for idx, item in enumerate(visible_items):
        real_idx = idx + offset
        if real_idx == selected_idx:
            mode = curses.A_REVERSE if active else curses.A_BOLD
        else:
            mode = curses.A_NORMAL
        win.addstr(idx + 1, 1, item[:w-3], mode)
    win.box()
    win.refresh()



def draw_bottom_bar(stdscr):

    curses.start_color()
    curses.init_pair(1, curses.COLOR_BLACK, curses.COLOR_CYAN)  # text black, background cyan

    h, w = stdscr.getmaxyx()
    #
    #
    bar_text = " 2 Export  (s-)3 View  4 Insert  5 Copy  6 Move/RenameDB  7 Create  8 Delete "
    hostname = socket.gethostname()
    # Reserve space for hostname plus a space before it
    max_bar_width = w - len(hostname) - 1
    bar_text = bar_text[:max_bar_width]
    bar_text = bar_text.ljust(max_bar_width) + " " + hostname
    #
    #
    stdscr.attron(curses.color_pair(1))
    #stdscr.attron(curses.A_REVERSE)

    # Truncate bar_text if wider than screen width
    text = bar_text[:w]
    try:
        stdscr.addstr(h-1, 0, text.ljust(w))
    except curses.error:
        # In case window too small, ignore error
        pass
    #stdscr.attroff(curses.A_REVERSE)
    stdscr.attroff(curses.color_pair(1))
    stdscr.refresh()

def input_dialog(stdscr, prompt):
    curses.echo()
    h, w = stdscr.getmaxyx()
    win = curses.newwin(3, w//2, h//2 - 1, w//4)
    win.box()
    win.addstr(1, 2, prompt)
    win.refresh()
    stdscr.refresh()
    curses.curs_set(1)
    input_win = curses.newwin(1, w//2 - len(prompt) - 4, h//2, w//4 + len(prompt) + 2)
    curses.echo()
    input_win.clear()
    input_win.refresh()
    s = input_win.getstr().decode('utf-8')
    curses.noecho()
    curses.curs_set(0)
    return s

# ================================================================================
#    MAIN *****************************
# --------------------------------------------------------------------------------

@click.command()
@click.option('--logfile', default='/tmp/mcinflux_debug.log', help='Log file path')
@click.option('--user', "-u", is_flag=True, help='Act as user, use user name and pass')
def main(logfile, user):

    global user_glob


    is_there_new_version(package="jusfltuls", printit=True, printall=True)

    print("Hey, machine is expected in the format like '127.0.0.1' or 'www.example.com' (where --ssl is selected) ")
    logging.basicConfig(filename=logfile, level=logging.DEBUG,
                        format='%(asctime)s - %(levelname)s - %(message)s')
    logging.debug('Debug log started')

    print("""
---------------------------------------------------------------------------
THE FILE(S) NEEDED:""")
    maninf.verify_files(justprint=True)
    print()

    print(f"i... switch 'user' is {user}")
    maninf.user_glob = user # set this for all session
    print(f"i... global var 'userg' is {maninf.user_glob}")
    #
    ok = maninf.verify_files()
    if not ok:
        print("X...  files missing")
        sys.exit(1)
    #
    u, p, serv =maninf.get_user_pass()
    if u is None or p is None:
        print("X... user o pass problem", u)
        sys.exit(1)
    #else:        print(f"/{u}/, /{p}/")

    m =serv#maninf.get_local_machine()

    if m is None:
        print("X... machine name problem")
        sys.exit(1)

    print(f"i... user: {fg.green}{u}{fg.default} on machine: {fg.yellow}{m}{fg.default} , I am going  there")
    res = maninf.show_databases()
    if len(res) == 0:
        print(f"X... NO DATABASES seen:   trying -u switch, {fg.magenta} changing user {fg.default} ")
        time.sleep(1)
        #
        maninf.user_glob = not user # set this for all session
        print(f"i... variable 'userg' is {maninf.user_glob}")
        #
        u, p, serv =maninf.get_user_pass()
        if u is None or p is None:
            print("X... user or passwd problem")
            maninf.is_infl_active()
            sys.exit(1)
        #else:        print(f"/{u}/, /{p}/")

        m =serv#maninf.get_local_machine()

        if m is None:
            print("X... machine name problem")
            sys.exit(1)

            print(f"i... user: {u} on machine: {m} , I am going  there")


    print("""
    TO CREATE A USER AND PRIVILEGE:
    nano /etc/influxdb/influxdb.conf ; auth-enabled = false ;
    influx ; create user abc with password 'def' with all privileges;
    nano /etc/influxdb/influxdb.conf; auth-enabled = true;
    systemctl restart influxd
    ---------------------------------------------------------------------------
""")
    maninf.verify_files()

    def curses_main(stdscr):
        #print("Ciao")
        logging.debug("Ciao")
        left_db_selected = None
        right_db_selected = None

        # previous idx==remember where to return
        left_idx_prev = 0
        right_idx_prev = 0

        left_offset, right_offset = 0, 0

        curses.curs_set(0)
        stdscr.clear()
        stdscr.refresh()
        left_items = get_left_panel_status()
        right_items = get_right_panel_status()
        left_idx, right_idx = 0, 0
        active_panel = 'left'

        h, w = stdscr.getmaxyx()
        mid = w // 2

        left_win = curses.newwin(h-1, mid, 0, 0)
        right_win = curses.newwin(h-1, w - mid, 0, mid)


        def refresh_panels():
            nonlocal left_items, right_items, left_idx, right_idx
            if left_db_selected is None:
                left_items = get_left_panel_status()
            else:
                left_items = ['..'] + maninf.show_measurements(left_db_selected)
            if right_db_selected is None:
                right_items = get_right_panel_status()
            else:
                right_items = ['..'] + maninf.show_measurements(right_db_selected)
            left_idx = min(left_idx, len(left_items) - 1) if left_items else 0
            right_idx = min(right_idx, len(right_items) - 1) if right_items else 0

        refresh_panels()

        # older
        #draw_panel(left_win, left_items, left_idx, active_panel == 'left')
        #draw_panel(right_win, right_items, right_idx, active_panel == 'right')
        # newer with scrolling
        draw_panel(left_win, left_items, left_idx, active_panel == 'left', left_offset)
        draw_panel(right_win, right_items, right_idx, active_panel == 'right', right_offset)
        #
        draw_bottom_bar(stdscr)

        while True:
            key = stdscr.getch()
            if key == 9:  # TAB
                active_panel = 'right' if active_panel == 'left' else 'left'
                if active_panel == 'left':
                    left_idx = max(0, min(left_idx, len(left_items) - 1))
                else:
                    right_idx = max(0, min(right_idx, len(right_items) - 1))

            # ---------------------------------------- original movement in panels

            # elif key == curses.KEY_UP:
            #     if active_panel == 'left':
            #         left_idx = max(0, left_idx - 1)
            #     else:
            #         right_idx = max(0, right_idx - 1)

            # elif key == curses.KEY_DOWN:
            #     if active_panel == 'left':
            #         left_idx = min(len(left_items) - 1, left_idx + 1)
            #     else:
            #         right_idx = min(len(right_items) - 1, right_idx + 1)


            # new panel movement with scrolling
            elif (key == curses.KEY_UP) and active_panel == 'left':
                if left_idx > 0:
                    left_idx -= 1
                    if left_idx < left_offset:
                        left_offset -= 1
            elif (key == curses.KEY_DOWN) and active_panel == 'left':
                if left_idx < len(left_items) - 1:
                    left_idx += 1
                    h, _ = left_win.getmaxyx()
                    if left_idx >= left_offset + h - 2:
                        left_offset += 1
            elif (key == curses.KEY_UP) and active_panel == 'right':
                if right_idx > 0:
                    right_idx -= 1
                    if right_idx < right_offset:
                        right_offset -= 1
            elif (key == curses.KEY_DOWN) and active_panel == 'right':
                if right_idx < len(right_items) - 1:
                    right_idx += 1
                    h, _ = right_win.getmaxyx()
                    if right_idx >= right_offset + h - 2:
                        right_offset += 1



            # ------------------------------------------------------------
            elif key == ord('2'):# --------------------------------- VIEW MEAS
                name = None
                dbase = None
                if active_panel == 'left':
                    name = left_items[left_idx]
                    dbase = left_db_selected
                    other_dbase = right_db_selected
                else:
                    name = right_items[right_idx]
                    dbase = right_db_selected
                    other_dbase = left_db_selected

                res = maninf.export_measurement(name, dbase, silent=True)
                logging.debug(f"{res}")
                #show_message_box(stdscr, f"{res}")
                refresh_panels()

            elif key == ord('3'):# --------------------------------- VIEW MEAS
                name = None
                dbase = None
                if active_panel == 'left':
                    name = left_items[left_idx]
                    dbase = left_db_selected
                    other_dbase = right_db_selected
                else:
                    name = right_items[right_idx]
                    dbase = right_db_selected
                    other_dbase = left_db_selected

                res = maninf.show_measurement_newest_sample(name, dbase, silent=True)
                logging.debug(f"{res}")
                show_message_box(stdscr, f"{res}")
                refresh_panels()

            elif key == ord('#'):#   shift-3------------------- VIEW MEAS
                name = None
                dbase = None
                if active_panel == 'left':
                    name = left_items[left_idx]
                    dbase = left_db_selected
                    other_dbase = right_db_selected
                else:
                    name = right_items[right_idx]
                    dbase = right_db_selected
                    other_dbase = left_db_selected

                logging.debug(f"repeating view ....")
                stdscr.nodelay(True)  # Non-blocking input
                time_restrict = dt.datetime.now().isoformat() + "Z"  # ISO 8601 format with Zulu time
                #
                local = dt.datetime.now().astimezone()  # local time with timezone info
                utc_dt = local.astimezone(dt.timezone.utc)
                time_restrict = utc_dt.isoformat().replace("+00:00", "Z")
                #time_restrict = dt.datetime.now().strftime("%Y-%m-%d %H:%M:%S")#isoformat() + "Z"  # ISO 8601 format with Zulu time
                time1 = dt.datetime.now().strftime("%H:%M:%S")
                logging.debug(f"repeating view ....{time_restrict}")

                while True:
                    time2 = dt.datetime.now().strftime("%H:%M:%S") # interesting. when time1 inside strftime, overrided
                    res = maninf.show_measurement_newest_sample(name, dbase, silent=True, prepend=f"measurement from {time1}  ==>>>  {time2}", time_restrict=time_restrict)
                    show_message_box(stdscr, f"{res}", wait_key=False)
                    time.sleep(2)
                    newkey = -1
                    ####newkey = stdscr.getch()
                    try:
                        newkey = stdscr.getch()
                    except curses.error:
                        newkey = -1
                    if newkey != -1:  # Any key pressed
                        break
                    refresh_panels()
                    draw_panel(left_win, left_items, left_idx, active_panel == 'left', left_offset)
                    draw_panel(right_win, right_items, right_idx, active_panel == 'right', right_offset)
                    #
                    draw_bottom_bar(stdscr)

                stdscr.nodelay(False)  # Non-blocking input
                refresh_panels()

            elif key == ord('4'):# --------------------------------- INSERT MEAS
                name = None
                dbase = None
                if active_panel == 'left':
                    name = left_items[left_idx]
                    dbase = left_db_selected
                    other_dbase = right_db_selected
                else:
                    name = right_items[right_idx]
                    dbase = right_db_selected
                    other_dbase = left_db_selected

                aaa = ''.join(chr(random.randint(97, 122)) for _ in range(3))
                res = maninf.insert_new_measurement(f"z_test_{aaa}", dbase, silent=True)
                logging.debug(f"{res}")
                #show_message_box(stdscr, f"{res}")
                refresh_panels()

            elif key == ord('5'):# --------------------------------- COPY MEAS
                name = None
                dbase = None
                if active_panel == 'left':
                    name = left_items[left_idx]
                    dbase = left_db_selected
                    other_dbase = right_db_selected
                else:
                    name = right_items[right_idx]
                    dbase = right_db_selected
                    other_dbase = left_db_selected
                on_copy(name, dbase, other_dbase, stdscr)
                refresh_panels()
            elif key == ord('6'):# --------------------------------- MOVE MEAS
                name = None
                dbase = None
                if active_panel == 'left':
                    name = left_items[left_idx]
                    dbase = left_db_selected
                    other_dbase = right_db_selected
                else:
                    name = right_items[right_idx]
                    dbase = right_db_selected
                    other_dbase = left_db_selected
                if dbase is None or other_dbase is None:
                    logging.debug(f"I am in db view and F6 db=={name} this time. TO RENAME")
                    newname = input_dialog(stdscr, f"Rename {name} to : ")
                    if len(newname) > 0:
                        maninf.create_database(newname)
                        # now copy  all measurements in
                        res = maninf.show_measurements(name)
                        logging.debug(f"About to copy this to {newname}: {res}")
                        for i in res:
                            logging.debug(f" - copying {i} from {name} to {newname}")
                            on_copy(i, name, newname, stdscr, asking=False)
                            logging.debug(f" - not dropping {i} in {name}; ")
                        name = input_dialog(stdscr, f"You drop {name} database yourself ")
                    else:logging.debug("empty string for newname")
                else:
                    logging.debug(f"I am in measurement view and F6 mea=={name}")
                    on_copy(name, dbase, other_dbase, stdscr, wordcopy="MOVE")
                    on_delete_measurement(name, dbase, stdscr, asking=False) # in move
                refresh_panels()
                #on_move()
                #refresh_panels()
            elif key == ord('7'):# --------------------------------- CREATE
                name = input_dialog(stdscr, "Create name: ")
                on_create_database(name)
                refresh_panels()
            elif key == ord('8'): # --------------------------------- DELETE
                logging.debug("8... in delete mode...")
                if active_panel == 'left':
                    if left_items:
                        #logging.debug("8... deleting...left...")
                        #logging.debug(f"8... deleting...left...{left_idx}")
                        selected_item = left_items[left_idx]
                        logging.debug(f"8. deleting...selected: {selected_item}, db={left_db_selected}")
                        if (selected_item != '..') and (left_db_selected is None): #  view ??? meas?
                            logging.debug(f"*in delete DB and db_sel {selected_item}")
                            #name = left_items[left_idx]
                            on_delete_database(selected_item, stdscr)
                            #on_delete_measurement(name, left_db_selected, stdscr)
                        elif (left_db_selected is not None): #  view datab ???
                            name = left_items[left_idx]
                            # name== selected_item
                            logging.debug(f"*in delete MEAS and Me={name} sel=={selected_item} DB={left_db_selected}")
                            on_delete_measurement(selected_item, left_db_selected, stdscr)
                if active_panel == 'right':
                    if right_items:# right panel  ***********************
                        logging.debug("8... deleting...right ... TO DO !!!")
                        selected_item = right_items[right_idx]
                        logging.debug(f"8. deleting...selected: {selected_item}, db={right_db_selected}")
                        if (selected_item != '..') and (right_db_selected is None): #  view ??? meas?
                            logging.debug(f"*in delete DB and db_sel {selected_item}")
                            on_delete_database(selected_item, stdscr)
                        elif (right_db_selected is not None): #  view datab ???
                            name = right_items[right_idx]
                            logging.debug(f"*in delete MEAS and Me={name} sel=={selected_item} DB={right_db_selected}")
                            on_delete_measurement(selected_item, right_db_selected, stdscr)
                # -----------------------------------------------------
                refresh_panels()

            elif key == ord('r'): # --------------------------------- refresh
                refresh_panels()

            elif key == ord('b'): # --------------------------------- backup
                on_backup(stdscr)

            elif key in (curses.KEY_ENTER, 10, 13):
                if active_panel == 'left':
                    logging.debug(f"Left ENTER db-selected=={left_db_selected}")
                    if left_items:
                        selected_item = left_items[left_idx]
                        logging.debug(f"Left ENTER item-selected=={selected_item}")
                        if selected_item == '..': # going up
                            left_db_selected = None
                            left_items = get_left_panel_status()
                            left_idx = left_idx_prev # not 0
                        elif left_db_selected is None:
                            left_db_selected = selected_item
                            left_idx_prev = left_idx # store  previous index
                            left_items = ['..'] + maninf.show_measurements(left_db_selected)
                            left_idx = 0
                        else:
                            logging.debug(f"enter on measurement: see newest; m1={selected_item} db1={left_db_selected}")
                            res = maninf.show_measurement_newest_oldest(selected_item, left_db_selected, silent=True)

                            logging.debug(f"{res}")
                            show_message_box(stdscr, res)
                            pass
                else:
                    logging.debug(f"Right ENTER db-selected=={right_db_selected}")
                    if right_items:
                        selected_item = right_items[right_idx]
                        if selected_item == '..':
                            right_db_selected = None
                            right_items = get_right_panel_status()
                            right_idx = right_idx_prev # 0
                        elif right_db_selected is None:
                            right_db_selected = selected_item
                            right_idx_prev = right_idx # store
                            right_items = ['..'] + maninf.show_measurements(right_db_selected)
                            right_idx = 0
                        else:
                            logging.debug(f"enter on measurement: see newest; m1={selected_item} db1={right_db_selected}")
                            res = maninf.show_measurement_newest_oldest(selected_item, right_db_selected, silent=True)

                            logging.debug(f"{res}")
                            show_message_box(stdscr, res)
                            pass

            # elif key in (curses.KEY_ENTER, 10, 13):
            #     if active_panel == 'left':
            #         if left_items:
            #             left_db_selected = left_items[left_idx]
            #             left_items = maninf.show_measurements(left_db_selected)
            #             left_idx = 0
            #     else:
            #         if right_items:
            #             right_db_selected = right_items[right_idx]
            #             right_items = maninf.show_measurements(right_db_selected)
            #             right_idx = 0

            elif key in (ord('q'), 27):  # q or ESC to quit
                break

            #refresh_panels()
            # older
            #draw_panel(left_win, left_items, left_idx, active_panel == 'left')
            #draw_panel(right_win, right_items, right_idx, active_panel == 'right')
            #  newer with scrolling
            draw_panel(left_win, left_items, left_idx, active_panel == 'left', left_offset)
            draw_panel(right_win, right_items, right_idx, active_panel == 'right', right_offset)
            #
            draw_bottom_bar(stdscr)

    curses.wrapper(curses_main)

if __name__ == "__main__":
    main()



#!/usr/bin/env python3


# -----------  jsut checking number of parameters.
# --- this script is called from py wrapper
# --- and paramters are passed using sys.argv
# --- and compatible with uv uvx
#

import subprocess as sp
import shlex
from fire import Fire
import os
import sys

from jusfltuls.check_new_version import is_there_new_version

def main():
    is_there_new_version(package="jusfltuls", printit=True, printall=True)

    command=f"smartnow.sh"

    if len(sys.argv) < 2:
        print("In python wrapper: Usage: smartnow <par> [par2 ...]")
    else:
        for i in sys.argv[1:]:
            command = f"{command} {i}"

    script_dir = os.path.dirname(os.path.abspath(__file__))
    print("D... ...", script_dir)
    full_command = os.path.join(script_dir, command)
    print("D... ...", full_command)
    CMD = shlex.split(full_command)
    # IF it is interactive... FULL INTERACTIVITY    result = sp.run( CMD )
    # ELSE
    result = sp.run( CMD, capture_output=True, text=True )
    if result.returncode != 0:
        print("X... error calling", command)
    print(result.stdout ) ####   REPOERT THE OUTPUT
    return
if __name__ == "__main__":
    Fire(main)



#!/usr/bin/env python3

#from pyfiglet import Figlet
from fire import Fire
import subprocess as sp
import os
import time
import sys
import random

from jusfltuls.check_new_version import is_there_new_version

#
#   NEEDS apt install toilet
#
#


lastp = []
lastp2 = []
lastp3 = []
MIN60 = 60


def get_uni(i, leng=False):
    uni=['\u2583','\u2584',
         '\u2585','\u2586',
         '\u2586','\u2587']
    if leng:
        # random number 1 .. len-1
        r = random.randint(0,len(uni)-2)+1
        #print("RANDOM",r)
        return r
    #print(i, len(uni) )
    k=i #% len(uni)
    #k=random.randint(0,k)
    return uni[k].encode("utf16","surrogatepass").decode("utf16")


def ping(who="192.168.0.1"):
    """
    one ping
    """
    global lastp,lastp2,lastp3
    CMD = "ping -w 1 -i 1 "+who
    l = get_uni(0, leng=True)
    try:
        res = sp.check_output(CMD.split()).decode("utf8")
        ok = "green"
        lastp.append(l)
        lastp2.append(l)
        lastp3.append(l)
    except:
        time.sleep(1)
        ok = "red"
        lastp.append(0)
        lastp2.append(0)
        lastp3.append(0)
    #print(lastp)
    #output(who,ok)
    return who,ok


def bar(  n=10 ):
    """
    do not run
    """
    global lastp,lastp2,lastp3
    while len(lastp)>n:
        lastp.pop(0)
    i = 0
    for color in lastp:
        i+=1
        if color==0:
            print('\033[0;31m', end="", flush=True)
        else:
            print('\033[0;32m', end="", flush=True)
        CHAR = get_uni( color )
        print(CHAR, flush=True, end="")
        if i>n: break
    print()

    #-----------second bar--------------
    i = 0
    suma = 0
    while len(lastp2)>MIN60*MIN60:
        lastp2.pop(0)
    for color in lastp2:
        i+=1
        if color!=0:color=1
        suma+= color
        if i>=MIN60:
            #print(i,suma)
            if suma==0:
                print('\033[0;31m', end="", flush=True)
            elif suma>=MIN60:
                print('\033[0;32m', end="", flush=True)
            else:
                print('\033[0;35m', end="", flush=True) #33ora 34blu
            CHAR = get_uni( 1 )
            print(CHAR, end="", flush=True)
            i = 0
            suma=0

    print()


    #-----------3rd bar--------------
    i = 0
    suma = 0
    while len(lastp3)>MIN60*MIN60*MIN60:
        lastp3.pop(0)
    for color in lastp3:
        i+=1
        if color!=0:color=1
        suma+= color
        if i>=MIN60*MIN60:
            if suma==0:
                print('\033[0;31m', end="", flush=True)
            elif suma==MIN60:
                print('\033[0;32m', end="", flush=True)
            else:
                print('\033[0;35m', end="", flush=True)
            CHAR = get_uni( 3 )
            print(CHAR, end="", flush=True)
            i = 0
            suma=0

    print()


def main():
    """
    indefinite ping wth minute and hour bars
    """

    is_there_new_version(package="jusfltuls", printit=True, printall=True)
    if len(sys.argv) < 2:
        print("Usage: pingy <addr>")
        sys.exit(1)
    addr = sys.argv[1]
    rang = MIN60

    for i in range(3600*24*7):
        #print("")
        time.sleep(0.1)
        #print(i)
        # ping fills lastp
        who,ok = ping(addr)
        os.system("clear")
        #print('\033[1;1H' + time.asctime(time.localtime()))
        print('\033[1;1H')
        output(who,ok)
        #print('\033[1;1H', " "*rang)
        # bar prints 1 bar
        bar( n = rang)
        #time.sleep(1)


def output(text="192.168.0.111", color="green"):
    """
    one shot ping
    """
    #f = Figlet(font='doom')  # too empty
    #f = Figlet(font='basic') # not clear
    #f = Figlet(font='colossal') # too large
    #f = Figlet(font='roman')  # too large
    #f = Figlet(font='univers') # too larrge
    #f = Figlet(font='rectangles') # too narrow
    #f = Figlet(font='computer') #worse than nancyj
    #f = Figlet(font='letters')

    #f = Figlet(font='nancyj') # better than letters

    CMD = "toilet -f mono9 "+text
    CMD = "toilet -f pagga "+text
    if color=="red":
        print('\033[0;31m', end="", flush = True)
        CMD = CMD + " f"
    if color=="green":
        print('\033[0;32m', end="", flush = True)
        CMD = CMD + " +"

    res = sp.check_call( CMD.split() )


    #print( f.renderText(text) )
    #print( res )

    #print( f.renderText("192 . 168 . 0 . 111 OK") )
    print('\033[0m', end="")

if __name__=="__main__":
    # Fire( {"o":output,
    #        "b":bar,
    #        "g":go,
    #        "p":ping})
    Fire( main )



def main() -> str:
    print("""
    dtox        - detox filenames with bad characters, but more useful
    cpuspeed    -
    pingy       -
    sshconf     -
    zoter       -
    wavescan    -
    smartnow    -
    jusfltuls   -
    distcheck   - helper for uv pushlishing in PyPI
    pycompress  -
    mci         - influx mc client
    ssshow      - services show
    fawhis      - fast_whisper
    mpvsa       - mpv, but for a video, find the best audio and subtitle in the folder; create waterfall
    """)
    return "Hello from usfltuls! Myself, I reside in __init__; TRY sshconf dtox cpuspeed pingy smartnow distcheck"



#!/usr/bin/env python3

import subprocess as sp
import os
import shlex
from console import fg, bg, fx
import click
import datetime as dt
from configparser import ConfigParser
import socket


user_glob = True


def is_int(n):
    if str(n).find(".")>=0:  return False
    if n is None:return False
    try:
        float_n = float(n) # 0.0
        int_n = int(float_n) #0.0
    except ValueError:
        return False
    else:
        return float_n == int_n


def verify_files( justprint=False):
    ok = True
    #FIS = ["~/.config/influxdb/username_local_admin",
    #    "~/.config/influxdb/password_local_admin",
    #    "~/.config/influxdb/username_local",
    #    "~/.config/influxdb/password_local",
    #    "~/.config/influxdb/server_local"]
    FIS = ["~/.config/influxdb/totalconfig.conf"]
    for i in FIS:
        if justprint:
            print( i)
        else:
            FF = os.path.expanduser(i)
            if not os.path.exists(FF):
                ok = False
                print(f"X... {FF} doesnt exist")
    return ok

def get_user_pass():
    global user_glob
    user = user_glob # must be set
    FIS = ["~/.config/influxdb/totalconfig.conf"]

    config = ConfigParser()
    config.read( os.path.expanduser(FIS[0]) )
    #print( dict(config)  )
    u, p, s = None, None, None
    if "influx local" in config.sections():
        if user:
            u = config["influx local"]['username']
            p = config["influx local"]['password']
            s = config["influx local"]['server']
        else:
            u = config["influx local"]['username_admin']
            p = config["influx local"]['password_admin']
            s = config["influx local"]['server']
        #print(u, p, s)
    return u, p, s
    # #print(f"D... user in gup is {user}")
    # u, p, d = None, None, None
    # if user:
    #     FIu = "~/.config/influxdb/username_local"
    #     FIp = "~/.config/influxdb/password_local"
    #     FIu = os.path.expanduser(FIu)
    #     FIp = os.path.expanduser(FIp)
    #     if os.path.exists(FIu) and os.path.exists(FIp):
    #         with open(FIu) as f:
    #             res = f.readlines()
    #         u = res[0].strip()
    #         with open(FIp) as f:
    #             res = f.readlines()
    #         p = res[0].strip()
    #         d = "test"
    #     # FI = "~/.influx_userpassdb"
    #     # FI = os.path.expanduser(FI)
    #     # with open(FI) as f:
    #     #     res = f.readlines()
    #     # u = res[0].strip()
    #     # p = res[1].strip()
    #     # d = res[2].strip()
    # else:
    #     FIu = "~/.config/influxdb/username_local_admin"
    #     FIp = "~/.config/influxdb/password_local_admin"
    #     FIu = os.path.expanduser(FIu)
    #     FIp = os.path.expanduser(FIp)
    #     if os.path.exists(FIu) and os.path.exists(FIp):
    #         with open(FIu) as f:
    #             res = f.readlines()
    #         u = res[0].strip()
    #         with open(FIp) as f:
    #             res = f.readlines()
    #         p = res[0].strip()
    #         d = "test"
    return u, p, d

# def get_local_machine():
#     """
#     Sometimes:   -host  www.xxx.com  is needed  with --ssl
#     # -host 127.0.0.1
#     # --ssl -host http://www.ccom:8086
#     """
#     m = None
#     FIm = "~/.config/influxdb/server_local"
#     FIm = os.path.expanduser(FIm)
#     if os.path.exists(FIm):
#         with open(FIm) as f:
#             res = f.readlines()
#         m = res[0].strip()
#         if m.find("https://") >= 0:
#             m = m.replace("https://", "")
#         elif m.find("http://") >= 0:
#             m = m.replace("http://", "")
#         elif m.find(":8086") >= 0:
#             m = m.replace(":8086", "")
#         if m.find("/") >= 0:
#             m = m.replace("/", "")

#     return m


# ============================================================
#  check if influx active
# ------------------------------------------------------------
def is_infl_active():
    CMD = f"systemctl is-active influxd"
    CMDx = shlex.split(CMD)
    result = sp.run(CMDx, capture_output=True, text=True )
    if result.returncode != 0:
        print("!...  PROBLEM with execution", CMDx)
        ok = False
    else:
        print("i... influxd seems to  be : ", result.stdout)



# ================================================================================
#          CALL  COMMAND
# --------------------------------------------------------------------------------
def call_cmd(CMD2, database=None, fromdb=None, todb=None, silent=False, format="", rfc=False):
    u, p, s = get_user_pass()
    m = s#get_local_machine()
    CMD = ""
    ssl = ""
    rfc3339 = ""
    if rfc:
        rfc3339 = "-precision rfc3339"
    if not is_int(m.split(".")[-1]):
        ssl = "--ssl"
    if database is None:
        CMD = f"influx {rfc3339} -username '{u}' -password '{p}' {ssl} -host {m} -execute '{CMD2}' {format}"
    else:
        CMD = f"influx {rfc3339} -username '{u}' -password '{p}'  {ssl} -host {m}  -database {database} -execute \"{CMD2}\" {format}"
        #print(CMD)
    CMDx = shlex.split(CMD)
    result = sp.run(CMDx, capture_output=True, text=True )
    if result.returncode != 0:
        print("!...  CMD :(=", CMDx, "#")
        ok = False
    else:
        if not silent:
            print(fg.dimgray, f"{result.stdout}", fg.default)
    return result.stdout


# ================================================================================
# EXPORT                separate call
# --------------------------------------------------------------------------------

# def export_measurement(m1, db1, silent=False):
#     print(fg.blue, f"export new measurement {m1} to {db1} ", fg.default)
#     u, p, _ = get_user_pass()
#     m = get_local_machine()
#     now = dt.datetime.now().strftime("%Y%m%d_%H%M")
#     outfilename = f"export_{db1}_{m1}_{now}.csv"
#     CMD = ""
#     ssl = ""
#     if not is_int(m.split(".")[-1]):
#         ssl = "--ssl"
#     #
#     CMD = f"influx -username '{u}' -password '{p}'  {ssl} -host {m}  -database {db1} -execute 'SELECT * FROM {m1}' -format csv"
#     #
#     with open(outfilename, "w") as f:
#         sp.run(shlex.split(CMD), stdout=f)
#     return "ok"

def export_measurement(m1, db1, silent=False):
    """
    per parts
    """
    import csv
    u, p, s = get_user_pass()
    m = s#get_local_machine()
    offset = 0
    limit = 1_000_000
    now = dt.datetime.now().strftime("%Y%m%d_%H%M")
    ssl = ""
    if not is_int(m.split(".")[-1]):
        ssl = "--ssl"
    pg = 0
    hostname = socket.gethostname()
    while True:
        pg += 1
        outfilename = f"export_{hostname}_{db1}_{m1}_{now}_{pg:02}.csv"
        print(f"\n{fg.yellow}i... saving {outfilename} {fg.default}")
        CMD = f"influx -username '{u}' -password '{p}' {ssl} -host {m} -database {db1} -execute 'SELECT * FROM {m1} ORDER BY time ASC LIMIT {limit} OFFSET {offset}' -format csv"
        result = sp.run(shlex.split(CMD), capture_output=True, text=True)
        if not result.stdout.strip():
            break
        with open(outfilename, "w") as f:
            f.write(result.stdout)
        offset += limit
    return "ok"
# ================================================================================
# backup                 separate call
# --------------------------------------------------------------------------------

def backup_portable(PATH):
    print(fg.blue, f"backing up", fg.default)
    CMD = f"influxd backup -skip-errors -portable  {os.path.expanduser(PATH)}/ "
    CMD = f"influxd backup  -portable  {os.path.expanduser(PATH)}/ "
    CMDx = shlex.split(CMD)
    with open("/tmp/mcdebu.log", "a") as f:
        f.write("\n\n")
        f.write(CMD + "\n")
        f.write("CMDx\n")
        f.write(f"{CMDx}\n")
    sp.run(CMD, shell=True)
    #---------------------------------------------------



def show_databases():
    print(fg.blue, "... showing database ...", fg.default)
    res = call_cmd( " SHOW DATABASES " )
    #print(res)
    res = res.split("\n")[3:]
    print(res, ":")
    res = [ x for x in res if (len(x) > 0) and (x[0] != "_") and (x[0] != "-")  and (x.find("name") < 0)and (x.find("i_am_") < 0)  ]
    print(res)
    return res

def create_database( db ):
    print(fg.blue, "creating database", fg.default)
    res = call_cmd( f"  CREATE DATABASE '{db}' " ).strip().split("\n")
    #print(res, ":")
    #res = [ x for x in res if (x[0] != "_") and (x[0] != "-")  and (x.find("name") < 0)and (x.find("i_am_") < 0)]
    print(res)

def drop_database( db ):
    print(fg.blue, "dropping database", fg.default)
    res = call_cmd( f"  DROP DATABASE '{db}' " ).strip().split("\n")
    #print(res, ":")
    #res = [ x for x in res if (x[0] != "_") and (x[0] != "-")  and (x.find("name") < 0)and (x.find("i_am_") < 0)]
    print(res)

def show_measurements(database):
    print(fg.blue, f"showing measurements @ {database}", fg.default)
    res = call_cmd( f"SHOW MEASUREMENTS  ", database=database)
    #
    res = res.split("\n")[3:]
    #print(res, ":")
    res = [ x for x in res if (len(x) > 0) and (x[0] != "_") and (x[0] != "-")  and (x.find("name") < 0)and (x.find("i_am_") < 0)  ]
    print(res)
    return res

def copy_measurement(m1, db1, db2, silent=False):
    if not silent:
        print(fg.blue, f"copy measurement {m1} from {db1} to {db2}", fg.default)
    CMD = f"SELECT * INTO {db2}..{m1} FROM {db1}..{m1} group by *"
    if not silent:
        print(CMD)
    res = CMD
    res = call_cmd( CMD, database=db2)
    if not silent:
        print(res)
    return res
    #res = call_cmd( f"SELECT * INTO {m1}..[{db2}] FROM {m1}..[{db1}] group by *", fromdb=db1, to_db=db2)
    #print(res)
#select * into Verhaeg_Energy..[measurement_name_destination] from Verhaeg_IoT..[measurement_name_source] group by *

def delete_measurement(m1, db1):
    print(fg.blue, f"delete measurement {m1} from {db1} ", fg.default)
    CMD = f"DROP MEASUREMENT {m1} "
    res = CMD
    res = call_cmd( CMD, database=db1)
    return res
    #res = call_cmd( f"SELECT * INTO {m1}..[{db2}] FROM {m1}..[{db1}] group by *", fromdb=db1, to_db=db2)
    #print(res)
#select * into Verhaeg_Energy..[measurement_name_destination] from Verhaeg_IoT..[measurement_name_source] group by *


# ================================================================================
# INSERT
# --------------------------------------------------------------------------------

def insert_new_measurement(m1, db1, silent=False):
    print(fg.blue, f"insert new measurement {m1} to {db1} ", fg.default)
    CMD = f"INSERT {m1},tag_key=tag_value  field_key=34"
    #res = CMD
    res = call_cmd( CMD, database=db1, silent=silent)
    return res


# ================================================================================
# COUNT
# --------------------------------------------------------------------------------

def count_measurement(m1, db1, silent=False):
    if not silent: print(fg.blue, f"count new measurement {m1} to {db1} ", fg.default)
    CMD = f"SELECT COUNT(*) FROM  {m1}"
    #res = CMD
    res = call_cmd( CMD, database=db1, silent=silent)
    return res


def show_measurement_newest_sample(m1, db1, silent=False, prepend=None, time_restrict=None):
    if not silent: print(fg.blue, f"newest measurement {m1} from {db1} ", fg.default)
    CMD = ""
    if time_restrict is None:
        CMD = f"SELECT * FROM {m1} ORDER BY time DESC LIMIT 10"
    else:
        #time_restrict = datetime.datetime.now().isoformat() + "Z"  # ISO 8601 format with Zulu time
        CMD = f"SELECT * FROM {m1} WHERE time > '{time_restrict}' ORDER BY time DESC LIMIT 10"
    res = call_cmd( CMD, database=db1, silent=silent, rfc=True)
    #res = res.split("\n")[3:]
    #res = [x for x in res if len(x) > 0] # last line
    #if not silent: print(res)
    if prepend != None:
        return f"{prepend}\n{res}"
    return res

def show_measurement_newest(m1, db1, silent=False):
    if not silent: print(fg.blue, f"newest measurement {m1} from {db1} ", fg.default)
    CMD = f"SELECT * FROM {m1} ORDER BY time DESC LIMIT 1"
    res = call_cmd( CMD, database=db1, silent=silent)
    if len(res.strip()) < 1:return None
    res = res.split("\n")[3:]
    res = [x for x in res if len(x) > 0] # last line
    if len(res) < 1:
        print("X... no data here/newest")
        return None
    res = res[0].split()[0] # t
    res = int(res)
    if not silent: print(res)
    timen = dt.datetime.fromtimestamp(res / 1e9)
    if not silent: print(timen)
    now = dt.datetime.now()
    age = (now - timen)
    if not silent: print("AGE", age)
    sage = str(age)[:-7]
    sdate = str(timen)[:-7]
    if not silent: print("AGE", age, "    ",  sage, sdate)
    return timen


def show_measurement_oldest(m1, db1, silent=False):
    if not silent: print(fg.blue, f"oldest measurement {m1} from {db1} ", fg.default)
    CMD = f"SELECT * FROM {m1} ORDER BY time ASC LIMIT 1"
    res = call_cmd( CMD, database=db1, silent=silent)
    if len(res.strip()) < 1:return None
    res = res.split("\n")[3:]
    res = [x for x in res if len(x) > 0] # last line
    if len(res) < 1:
        print("X... No Data here/oldest")
        return None
    res = res[0].split()[0] # t
    res = int(res)
    if not silent: print(res)
    timen = dt.datetime.fromtimestamp(res / 1e9)
    if not silent: print(timen)
    now = dt.datetime.now()
    age = (now - timen)
    sage = str(age)[:-7]
    sdate = str(timen)[:-7]
    if not silent: print("AGE", age, "    ", sage, sdate)
    return timen

def show_measurement_newest_oldest(m1, db1, silent=False):
    """
    """
    tn = show_measurement_newest(m1, db1, silent=True)
    to = show_measurement_oldest(m1, db1, silent=True)
    aaa = ""
    #aaa = count_measurement(m1, db1, silent=silent)
    #aaa = f"COUNT  = {aaa[3:].split()[-1]} "
    #
    if tn is None or to is None: return None
    period = str(tn - to)
    if period[-7] == ".":period = period[:-7]
    now = dt.datetime.now()
    age = now - tn
    sage = str(age)[:-7]
    ID = f" measurement {m1} of {db1}"
    # remove fractions if present
    stn = str(tn)
    if len(stn) > 21: stn = stn[:-7]
    sto = str(to)
    if len(sto) > 21: sto = sto[:-7]
    res = f"\n{ID}\n--------------------------\nAGE    = {sage}   \nPERIOD = {period}\nNEWEST = {stn}\nOLDEST = {sto}\n{aaa}"
    if not silent: print(res)
    return res



@click.command()
@click.argument('command')
@click.option('--name', '-n')
@click.option('--fromdb', '-f', default=None)
@click.option('--todb', '-t', default=None)
@click.option('--user', '-u', count=True)
def main(command, name, fromdb, todb, user):
    global user_glob
    user_glob = user
    print("Hi")
    if command == "sd":
        show_databases()
    elif command == "cd":
        if name is None:
            print("X... name is none")
        create_database( name )
    elif command == "sm":
        if name is None:
            print("X... name is none")
            return
        show_measurements( name )
    elif command == "cm":
        if name is None or fromdb is None or todb is None:
            print("X... name or db is none")
            return
        copy_measurement( name, fromdb, todb )
    elif command == "sn":
        if name is None or fromdb is None:
            print("X... name or db is none")
            return
        show_measurement_newest( name, fromdb )
    elif command == "so":
        if name is None or fromdb is None:
            print("X... name or db is none")
            return
        show_measurement_oldest( name, fromdb )
    elif command == "sno":
        if name is None or fromdb is None:
            print("X... name or db is none")
            return
        show_measurement_newest_oldest( name, fromdb )
    elif command == "sns":
        if name is None or fromdb is None:
            print("X... name or db is none")
            return
        show_measurement_newest_sample( name, fromdb )
    elif command == "im":
        if name is None or todb is None:
            print("X... name or db is none")
            return
        insert_new_measurement( name, todb )
    elif command == "bp":
        backup_portable('~/INFL' )

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
#
# OUTDATED/USING CHRONY       ##### ntpsec-ntpdate ntp
# FIREWALL CHECK
# sudo ufw allow proto tcp from 1.3 to any port 8000 # sudo ufw allow proto tcp from 1.3 to any port 8099
#  for mass copy to SMB
# ls -1tr | tail -1 | xargs -I III  smbclient -U user%pass  //x.x.x.x/DATA   -c 'put "III"'
#
#
#### import pyautogui
import numpy as np # convert screen (black) to array

from terminaltables import AsciiTable,SingleTable

from blessings import Terminal  #
import time
import datetime

import threading

import fire

import subprocess as sp
import shlex

import socket
import os

#import ray
from influxdb import InfluxDBClient
from threading import Thread
import sys


import copy # to copy dict !

# from fire import Fire
from PIL import ImageGrab

from colorclass import Color, toggles    #
from console import fg, bg, fx
pause_main = False




# it is updated in functions
t = Terminal()


real_width = 10 # i try to mesure the width of the table

# source of IPs
CONFFORI = os.path.expanduser("~/.ssh/config")
CONFF = os.path.expanduser("~/.ssh/config")


tab_src = [ ['n', 'host','user','hostname','label', 'inf', 'ntp', 'cam','vnc','mys','ser','bor','mgo'] ] # LOWER!
# I dont spam 5678 etc...anymore
tab_src = [ ['n', 'host','user','hostname','label', 'inf', 'ntp', 'cam'] ] # LOWER!
selected_n = '0'
alldics_origin = {}
alldics = {}
DEBUG = False



import socket


def launch_command(name=None):
    CMD = name#"sleep 7"
    args = shlex.split(CMD)
    env = os.environ.copy()
    process = sp.Popen(args, env=env, stdout=sp.DEVNULL, stderr=sp.DEVNULL)
    process.poll()



def get_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.settimeout(0)
    try:
        # doesn't even have to be reachable
        s.connect(('10.254.254.254', 1))
        IP = s.getsockname()[0]
    except Exception:
        IP = '127.0.0.1'
    finally:
        s.close()
    return IP
#print(get_ip())


#--------https://stackoverflow.com/questions/2408560/python-nonblocking-console-input
# ON ENTER
# CALL WITH CALLBACK FUNCTION AS A PARAMETER
class KeyboardThread(threading.Thread):

    def __init__(self, input_cbk = None, name='keyboard-input-thread'):
        self.input_cbk = input_cbk
        self.block = False
        super(KeyboardThread, self).__init__(name=name)
        self.start()

    def run(self):
        while True:
            self.input_cbk(input()) #waits to get input + Return

    def pause(self):
        self.block = True

    def unpause(self):
        self.block = False





def my_callback(inp):
    """
    Read input + enter from keyboard. In the thread.
    """
    global alldics, alldics_origin, selected_n, pause_main
    #evaluate the keyboard input
    r = "autored"
    ybg = "autobgyellow"
    b = "autoblack"
    w = "autowhite"


    print('You Entered: /{}/'.format(inp) , flush = True)
    #time.sleep(2)

    if inp == 'r': #### KEY ####
        load_resources( CONFF)
        return


    already_selected = False
    for i in alldics.keys():
        # remove color
        if alldics[i]['host'].find("*")>0:
            already_selected = True
            #print("D... ALREADY SELECTED", alldics[i]['host'])
        #print("callback",alldics[i]['host'])
        #alldics[i]['host'] = alldics[i]['host'].rstrip("*")
        alldics[i]['host']  = Color('{'+w+'}'+alldics_origin[i]['host'] +'{/'+w+'}') # ONE COLOR ONLY
        alldics[i]['label']  = Color('{'+w+'}'+alldics_origin[i]['label'] +'{/'+w+'}') # ONE COLOR ONLY
        # set color

    if not already_selected:
        for i in alldics.keys():
            # if alldics[i]['n']==inp:
            if alldics[i]['n'] in inp:
                #alldics[i]['host']+="*"
                alldics[i]['host']  = Color('{'+ybg+'}'+'{'+b+'}'+alldics_origin[i]['host'] +'*{/'+b+'}'+'{/'+ybg+'}') #yellow
                alldics[i]['label']  = Color('{'+ybg+'}'+'{'+b+'}'+alldics_origin[i]['label'] +'*{/'+b+'}'+'{/'+ybg+'}') #yellow
                selected_n = inp
        return



    #========================= GO THROUGH ALL SELECTED
    #========================= GO THROUGH ALL SELECTED
    #========================= GO THROUGH ALL SELECTED

    if selected_n != '0': #### if something i selected, go one by one.... ###

        # ----------precalculate screens ----------------
        screenset = []
        if "c" in inp:
            # img = ImageGrab.grab() ###  NOT WITH UBUNTU 22 AND WAYLAND... X11
            # xmaxw = img.size[0]
            # ymaxh = img.size[1]

            # -----variant with py
            #img = pyautogui.screenshot()
            #--------------------------------------------  REMOVING THIS GRAB
            #img = ImageGrab.grab()
            #img = np.array(img.convert("RGB"))
            ymaxh = 1920#img.shape[0]
            xmaxw = 1080#img.shape[1]

            nimg = len(selected_n)
            if xmaxw>(640*(nimg)):
                print(f"ONE ROW  {xmaxw} ...  #=={nimg} need=={640*nimg}" )
                print(f"ONE ROW  {xmaxw} ...  #=={nimg} need=={640*nimg}" )
                for i in range(nimg):
                    screenset.append( f"{(i)*640+1}x{1}")
            else:
                print("TWO ROWS for now")
                jaj=0
                for i in range(nimg):
                    if (i+1)*640<=xmaxw:# <=nimg/2:
                        jaj=i
                        screenset.append( f"{(i)*640+1}x{1}")
                    else:
                        screenset.append( f"{(i-jaj-1)*640+1}x{481}")

            print( f"{xmaxw} {screenset}")
            print( f"{xmaxw} {screenset}")
            print( f"{xmaxw} {screenset}")
            print( f"{xmaxw} {screenset}")



        #### clusterssh ##### and return after
        if (len(selected_n)>1) and ('s' in inp): #### KEY ####
            #print("D... ssh "+alldics_origin[i]['host'])
            CMD = "clusterssh "
            CMD = "mssh "
            for i in alldics.keys():
                if alldics[i]['n'] in selected_n:
                    if CONFF!=CONFFORI: # no xforwarding and things in the case of not.config
                        CMD = CMD + alldics_origin[i]['user']+"@"+alldics_origin[i]['hostname']+"  "
                    else:
                        CMD = CMD +alldics_origin[i]['host']+"  "
            CMD+=" &"
            print(CMD)
            os.system(CMD)
            #launch_command(CMD)
            return






        #### Why I dont have a problem with "s" ???
        # --------------------------------   "s" in  vs.  "s" == ----------------
        for i in alldics.keys():


            ### not cluster, single terminals
            if alldics[i]['n'] in selected_n:

                print(f"D... i am in selected dictkey {selected_n} , input {inp}")
                if inp == 'h': #### KEY ####
                    ####  #####
                    HELP = f"""HELP______________________________________
  inf ... check for influx - port 8086
  ntp ... check chronyc -h IP tracking  NOT /ntpdig -dc HOST/  d-time  in [ms]
        .  NOT install ntp ntpsec-ntpdate; ufw allow 123/udp on client
  cam ... check port 8000 - expect video
    (c) .  flashcam uni :8000/video -p ~/.flashcam_HOST
  vnc ... check for VNCserver on port 5600
    (v) .  vncviewer -passwd ~/.vnc/HOST
  mys ... see 5678 myservice
    (m) .  see cronvice NOT myservice infinite on HOST
"""
                    with open("/tmp/sshconf.help","w") as f:
                        f.write(HELP)
                    #os.system(f"terminator -e 'bash -c \"cat /tmp/sshconf.help ; sleep 25\" '")
                    CMD = f"xterm -e 'bash -c \"cat /tmp/sshconf.help ; sleep 25\" '"
                    launch_command(CMD)



                 ############ --------- ############
                 ############  ssh      ############
                 ############ --------- ############
                if 's' == inp: #### KEY ####
                    #print("D... ssh "+alldics_origin[i]['host'])

                    if CONFF!=CONFFORI: # no xforwarding and things in the case of not.config
                        CMD = "xterm -e 'bash -c \"ssh \""+alldics_origin[i]['user']+"\"@"+alldics_origin[i]['hostname']+"; \" '"
                    else:
                        CMD = "xterm -e 'bash -c \"ssh "+alldics_origin[i]['host']+"; \" '"
                    print(CMD)
                    #
                    launch_command(CMD)
                    #args = shlex.split(CMD) # env=env,
                    #process = sp.Popen(args,  stdout=sp.DEVNULL, stderr=sp.DEVNULL)
                    #process.poll()
                    #
                    #os.system(CMD)


                 ############ --------- ############
                 ############   SSH interm##########
                 ############ --------- ############
                if inp == 'S': #### KEY ####
                    #print("D... ssh "+alldics_origin[i]['host'])
                    ###kthread.pause()
                    pause_main = True
                    os.system("ssh "+alldics_origin[i]['host'] )
                    #CMD = "ssh "+alldics_origin[i]['host']
                    #launch_command(CMD) # NOTHERE!!!!!!!!!!
                    pause_main = False
                    ###kthread.unpause()





                               #### pingy ##### for ANY IN
                 ############ --------- ############
                 ############  pingy    ############
                 ############ --------- ############
                if 'p' in inp: #### KEY ####
#                    os.system("xterm -geometry 65x12 -e 'bash -c \"pingy.py  "+alldics_origin[i]['hostname']+"; \" '")
                    #os.system("terminator -e 'bash -c \"pingy.py  "+alldics_origin[i]['hostname']+"; \" '")
                    CMD = "xterm -e 'bash -c \"pingy     "+alldics_origin[i]['hostname']+"; \" '"
                    launch_command(CMD)


                                 #### rsync ~/DATA/ ##### for ANY IN
                 ############ --------- ############
                 ############  rsync    ############
                 ############ --------- ############
                if ('R' in inp): #### KEY ####
                    #print("D... rsync -av "+alldics_origin[i]['host'])
                    CMD = "rsync -av --progress "
                    CMD = CMD + alldics_origin[i]['user']+"@"+alldics_origin[i]['host']+":~/DATA/  "
                    CMD+=" ~/DATA/REMOTEDATA/ "

                    #CMD = f"xterm -geometry 100x20+10+600 -e \"bash -c ' {CMD} ; echo ALL OK; sleep 5' \" & "
                    CMD = f"xterm -geometry 100x20+10+600 -e 'bash -c \" {CMD} ; echo ALL OK; sleep 5\" '"
                    print(CMD)
                    launch_command(CMD)
                    #os.system(CMD)
                    # return # with clusterssh i had return not to mess up



                               #### camera ##### 5000
                 ############ --------- ############
                 ############  C 5000   ############
                 ############ --------- ############
                if 'C' == inp: #### KEY #### PORT 5000 !!
                    code = os.path.expanduser("flashcam")
                    code = os.path.expanduser("pogucam")

                    mXY=""
                    if len(screenset)>0: mXY=f"-X {screenset.pop(0)}"

                    PASFILE="~/.pycam_"+i
                    PASFILE="~/.config/flashcam/.flashcam_upw_"+i
                    PASFILE=os.path.expanduser(PASFILE)
                    if not os.path.isfile(PASFILE):
                        print("D... NO PASFILE ",PASFILE)
                        print("D... NO PASFILE ",PASFILE)
                        print("D... NO PASFILE ",PASFILE)
                        PASFILE=""
                    else:
                        PASFILE=" -q "+PASFILE

                    CMD = "terminator -e 'BASH_ENV=~/.bashrc bash -c \"  export PATH=$PATH:$HOME/.local/bin;  "+code+" uni http://"+alldics_origin[i]['hostname']+":5000/video  "+PASFILE+" \" ; sleep 5'"

                    CMD = "xterm -T "+alldics_origin[i]['hostname']+":5000  -geometry 80x5 -bg black -fg yellow -e 'BASH_ENV=~/.bashrc bash -c \" export PATH=$PATH:$HOME/.local/bin; "+code+" uni http://"+alldics_origin[i]['hostname']+f":5000/video {mXY} "+PASFILE+" \" ;  sleep 1' &"
                    print(CMD)
                    print(CMD)
                    print(CMD)
                    #os.system(CMD)
                    launch_command(CMD)

                    time.sleep(2)
                                  #### camera ##### 8000
                 ############ --------- ############
                 ############  c 8000   ############
                 ############ --------- ############
                elif 'c' in inp: #### KEY ####
                    # code = os.path.expanduser("~/02_GIT/GITLAB/pycamfw/better_imagezmq/izmq_send2.py")
                    # uniwrec.py disp -v http://...video # no /@end
                    # code = os.path.expanduser("~/02_GIT/GITLAB/pycamfw/better_imagezmq/uniwrec.py")
                    #
                    #code = os.path.expanduser("~/02_GIT/GITLAB/flashcam/bin_flashcam.py")
                    code = os.path.expanduser("flashcam")
                    code = os.path.expanduser("pogucam")
                    #

                    PASFILE="~/.pycam_"+i
                    PASFILE="~/.config/flashcam/.flashcam_upw_"+i
                    PASFILE=os.path.expanduser(PASFILE)
                    if not os.path.isfile(PASFILE):
                        print("D... NO PASFILE ",PASFILE)
                        print("D... NO PASFILE ",PASFILE)
                        print("D... NO PASFILE ",PASFILE)
                        PASFILE=""
                    else:
                        PASFILE=" -q "+PASFILE
                    # CMD = "terminator -e 'bash -c \""+code+" disp -v http://"+alldics_origin[i]['hostname']+":8000/video "+PASFILE+"; \" '"
                    mXY=""
                    if len(screenset)>0: mXY=f"-X {screenset.pop(0)}"
                    #                    CMD = "terminator -e 'BASH_ENV=~/.bashrc bash -c \" export PATH=$PATH:$HOME/.local/bin; "+code+" uni http://"+alldics_origin[i]['hostname']+f":8000/video {mXY} "+PASFILE+" \" ;  sleep 5'"

                    CMD = "xterm -T +"+alldics_origin[i]['hostname']+":8000 -geometry 80x5 -bg black -fg yellow -e 'BASH_ENV=~/.bashrc bash -c \" export PATH=$PATH:$HOME/.local/bin; "+code+" uni http://"+alldics_origin[i]['hostname']+f":8000/video {mXY} "+PASFILE+" \" ' &"
                    # POGUCAM
                    CMD = "xterm -T +"+alldics_origin[i]['hostname']+":8000 -geometry 80x5 -bg black -fg yellow -e 'BASH_ENV=~/.bashrc bash -c \" export PATH=$PATH:$HOME/.local/bin; "+code+"  http://"+alldics_origin[i]['hostname']+f":8000/video  \"  ' &"
                    print(CMD)
                    #print(CMD)
                    ##print(CMD)
                    #print(CMD)
                    #print(CMD)
                    #time.sleep(5)
                    #with open("debug","a") as f:
                    #    f.write(CMD)

                    os.system(CMD)
                    #launch_command(CMD)  # cannot do here with all this

                    time.sleep(2)





                #                #### vnc #####
                #  ############ --------- ############
                #  ############  vnc      ############
                #  ############ --------- ############
                # if 'v' in inp: #### KEY ####
                #     passwfile = os.path.expanduser('~/.vnc/'+alldics_origin[i]['host'])
                #     if os.path.isfile(passwfile):
                #         #os.system("terminator -e 'bash -c \"vncviewer -passwd "+passwfile+" "+alldics_origin[i]['hostname']+"; \" '")
                #         CMD = "xterm -e 'bash -c \"vncviewer -passwd "+passwfile+" "+alldics_origin[i]['hostname']+"; \" '"
                #         launch_command(CMD)

                #     else:
                #         #os.system("terminator -e 'bash -c \"echo create passw:; echo vncpass "+passwfile+"; sleep 1;vncviewer "+alldics_origin[i]['hostname']+"; \" '")
                #         CMD = "xterm -e 'bash -c \"echo create passw:; echo vncpass "+passwfile+"; sleep 1;vncviewer "+alldics_origin[i]['hostname']+"; \" '"
                #         launch_command(CMD)


                 ############ --------- ############
                 ############  cris     ############
                 ############ --------- ############
                #### myservice #####
                if 'm' in inp: #### KEY ####
                    # CMD = "xterm -geometry 100x20 -e 'bash -c \"ssh "+alldics_origin[i]['host']+" \"script /dev/null;myservice2 infinite;\"  \" '"
                    # CMD = "xterm -geometry 100x20 -e 'bash -c \"ssh "+alldics_origin[i]['host']+" myservice2 a;  \" '"
                    # CMD = "xterm -geometry 100x20 -e 'bash -c \"ssh "+alldics_origin[i]['host']+" myservice2 a; read; \" '"

                    # xterm -geometry 100x20 -e "bash -c \"ssh -t  ojr@janca screen -x myservice2_infinite; sleep 3\" "

                    CMD = "ssh -t "+alldics_origin[i]['host']+" ~/.local/bin/cronvice"
                    #CMD =  "xterm -geometry 100x20 -e \"bash -c ' " + CMD + " ' \" "
                    CMD =  "xterm -geometry 100x20 -e \"bash -c ' " + CMD + " ' \"  "



                    print("D... ", CMD )
                    #os.system( CMD)
                    launch_command(CMD)





#--------------------end callback--------------------
#--------------------end callback--------------------
#--------------------end callback--------------------
#--------------------end callback--------------------
#--------------------end callback--------------------
#--------------------end callback--------------------
#--------------------end callback--------------------






def user_from_scan22(host):
    CMD = f"nmap -sV {host} -p 22"
    res = sp.check_output(CMD.split()).decode("utf8")
    res = res.split("\n")
    res = [x for x in res if x.find("Ubuntu")>=0 ]
    res = [x for x in res if x.find("OpenSSH")>=0 ]
    # i am using ubuntu user for
    if len(res)>0 and res[0].find("8.9p1")>0:
        return "ubuntu"
    elif len(res)>0:
        return "pi"
    else:
        return "none"


def get_all_arp():
    CMD = f"arp -na"
    print(f"D... get_all_arp ... CMD=={CMD}", file = sys.stderr)
    try:
        res = sp.check_output(CMD.split()).decode("utf8")
        res = res.split("\n")
    except:
        print("X... arp problem ... are you on android?", file = sys.stderr)
        res=[]
    ddd = {}
    for i in res:
        if len(i.split())<3: break
        #print("D...",i, i.split())
        ip = i.split()[1]
        mac = i.split()[3]
        if ip in ddd.keys(): continue
        ip = ip.replace("(","")
        ip = ip.replace(")","")
        ddd[ip] = mac
    return ddd





#---------------------------------------------------------------------
def myip():
    return [l for l in ([ip for ip in socket.gethostbyname_ex(socket.gethostname())[2] if not ip.startswith("127.")][:1], [[(s.connect(("8.8.8.8", 53)), s.getsockname()[0], s.close()) for s in [socket.socket(socket.AF_INET, socket.SOCK_DGRAM)]][0][1]]) if l][0][0]

import ipaddress

def load_resources( param=""):
    """
    loads the data from ~/.ssh/config

    only if "#Label something" is in the record, it is taken into the table

    IF _given_ CONFIG exists => scans and creates and NEW NAME

    """
    global CONFF
    n = ['1','2','3','4','5','6','7','8','9','a','b','c','d','e','f','g','h','i','j','k','l','m','n','o','p','q','r','s','t','u','v','w','x','y','z']

    global alldics, alldics_origin, tab_src
    alldics = {}
    alldics_origin = {}

    if os.path.exists( os.path.expanduser( f"~/.ssh/{param}" ) ):
        CONFF = os.path.expanduser( f"~/.ssh/{param}" )
        print(f"i... file {CONFF} exists - I use a specific config")
        #sys.exit(0)


    if param == "arp":
        print("D... I do   arp scan")
        # me = myip()+"/24"
        CMD = f"arp -na"
        print(f"D... CMD=={CMD}")
        res = sp.check_output(CMD.split()).decode("utf8")
        res = res.split("\n")
        ddd = {}
        for i in res:
            if len(i.split())<3: break
            #print("D...",i, i.split())
            ip = i.split()[1]
            mac = i.split()[3]
            if ip in ddd.keys(): continue
            ip = ip.replace("(","")
            ip = ip.replace(")","")
            ddd[ip] = mac
        # ddd = sorted([(value,key) for (key,value) in ddd.items()])
        # ddd = sorted( (value,key) for (key,value) in ddd.items(), key = ipaddress.IPv4Address )

        # ddd = sorted(zip(ddd.values(), ddd.keys()),  key = ipaddress.IPv4Address )

        # ddd = {k: v for k, v in sorted(ddd.items(), key = ipaddress.IPv4Address, key=lambda item: item[1])}
        # sorted(ddd.values(), key = ipaddress.IPv4Address )
        MYCONFIG = ""
        mactaken = []
        for host in  sorted( ddd.keys() , key = ipaddress.IPv4Address ):  #
            if ddd[host].find("b8:27:eb")>=0:
                label = "raspberry 3/z"
                user = "pi"
            elif ddd[host].find("dc:a6:32:")>=0:
                label = "raspberry 4"
                user = "pi"
            else:
                continue
                user = "ooo"
                label = "some"
            if ddd[host] in mactaken:
                continue

            # get influx hostname

            remhost = host
            try:
                CMD = f'tdb_io ls infl  -ip {host}'
                res = sp.check_output(CMD.split()).decode("utf8")
                res = res.split("\n")
                res = [ x for x in res if x.find("i_am_")>=0]
                remhost = res[0].split("i_am_")[1]
            except:
                print(f"D... no influx on {host} OR tdb_io not installed")

            MYCONFIG+= f"\nHost {remhost}\n    HostName {host}\n    User {user}\n    #Label {label}\n"
            mactaken.append( ddd[host] )


        # print(res)
        print( MYCONFIG )
        #with open( os.path.expanduser("~/.ssh/config"),"w" ) as f:
        #    f.write(MYCONFIG)
        sys.exit(1)


    if param == "nmap":
        print("i... I do nmap ping scan, safe also on android ", file = sys.stderr)
        import socket
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect(("8.8.8.8", 80))
        h_name = s.getsockname()[0]
        print( "i... my IP address", h_name , file = sys.stderr )
        s.close()

        # import  scapy
        # from scapy.all import scapy,ARP,Ether,srp,arp ing
        # ans,unans = arping("192.168.0.0/24", verbose=0)
        # for s,r in ans:
        #     print("{} {}".format(r[Ether].src,s[ARP].pdst))


        # import socket
        # h_name = socket.gethostname()
        # print(h_name)
        # h_name = socket.gethostbyname(h_name)

        CMD = f"nmap -sn {h_name}/24"
        print(f"D... CMD=={CMD}" , file = sys.stderr)
        #sys.exit(0)
        res = sp.check_output(CMD.split()).decode("utf8")
        res = res.split("\n")
        #print(res)
        res = [x for x in res if (x.find("report for")>0) ]
        #mac = [x for x in res if (x.find("MAC Add")==0) ]

        print(res,  file = sys.stderr )
        #print(mac,  file = sys.stderr )

        hosts = [ x.split(" ")[-1].replace("(","").replace(")","") for x in res] # take IP and avoid ()
        print(hosts, file = sys.stderr)


        arps = get_all_arp()

        MYCONFIG = ""

        for host in hosts:
            label = "some"
            user = os.getlogin()  # my current user

            if host in arps:
                mac = arps[host]
                if mac.find("b8:27:eb")>=0:
                    label = "raspberry 3/z"
                    user = user_from_scan22(host)
                    #user = "pi"
                elif mac.find("dc:a6:32:")>=0:
                    label = "raspberry 4"
                    user = user_from_scan22(host)
                    #user = "pi"
            else:
                mac = ""

            remhost = host
            remhost2 = socket.getfqdn(host).split(".")[0]
            if not remhost2.isdigit():
                remhost = remhost2


            # try:
            #     CMD = f'tdb_io ls infl  -ip {host}'
            #     print(f"i...  {CMD}")
            #     res = sp.check_output(CMD.split()).decode("utf8")
            #     res = res.split("\n")
            #     res = [ x for x in res if x.find("i_am_")>=0]
            #     remhost = res[0].split("i_am_")[1]
            # except:
            #     print(f"D... no influx on {host} OR tdb_io not installed")

            MYCONFIG+= f"\nHost {remhost}\n    HostName {host}\n    User {user}\n    #Label {label}\n"

        print(MYCONFIG)
        sys.exit(0)

    #----------------- Original idea -  if config not present.... create it...
    if  (os.path.exists(CONFF))  and (os.stat(CONFF).st_size != 0):
        print("i... trying to open", CONFF)
        with open(CONFF) as f:
            lines = f.readlines()
        print("i... ok")
    else:
        print(f"!... NO ssh confi file exists. - try to create one:")
        print(f" ... nmap 192.168.0.1/24 ; sshconf.py nmap")
        print(f" ... parameter nmap  =={param}")

        user = os.environ.get('USER')
        ip = get_ip()
        newip = ".".join(ip.split(".")[:-1])

        machine = f"""
Host core6a
   HostName {newip+".22"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __

Host core6b
   HostName {newip+".23"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __

Host gigavg
   HostName {newip+".14"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __

Host filip
   HostName {newip+".17"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __

Host nasa
   HostName {newip+".20"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __

Host gigajm
   HostName {newip+".21"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __

Host tr24a
   HostName {newip+".15"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __


Host tr24b
   HostName {newip+".16"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __

Host kroha
   HostName {newip+".44"}
   IdentityFile ~/.ssh/id_rsa
   User {user}
   #Label: __


"""
        print(machine)
        sys.exit(0)




    lines = "".join(lines).split("Host ")
    lines = [ i.rstrip().lstrip().replace("  "," ") for i in lines]
    lines = [ i.replace("  "," ") for i in lines]
    lines = [ i.replace("  "," ") for i in lines]
    lines = [ i.replace("  "," ") for i in lines]

    while lines[0] == "":
        lines.pop(0)
    while lines[-1] == "":
        lines.pop()
    #print(lines)


    for i in lines: # Full record
        onedic = {} # EMPTY ONE LINE-RECORD  DICT
        its = i.split("\n")
        its = [i.strip() for i in its]

        onedic['host'] = its[0].lower()
        # loop for all fields in one record
        for j in its:

            for la in tab_src[0]: # Check the presence in tabsrc...HostName
                # here is wise trick to split by lowercase la
                # User/Label in FILE and user/label ... as a key in onedic
                # 20221219
                # I changed all MGO to mgo and the key will be always lower
                #  but it is not important
                #print(" >",la)
                if (j.lower().find( la )==0) or (j.lower().find( la )==1):
                    if la.lower()!='host':
                        #print( j.lower().split(la)[-1].strip()  )
                        #print( j.lower()  )
                        #print( "---")
                        #onedic[la.lower()] = j.lower().split(la)[-1].strip()
                        tojoin = j.strip().split(" ")[1:]
                        onedic[la.lower()] = " ".join(tojoin)

        # jump out---in no #LABEL
        if not 'label' in onedic:
            continue

        onedic['n'] = str(n[0])
        n.pop(0)
        if  onedic['host'] in alldics:
            print("X.. !!! host name already exists:", onedic['host'])
            sys.exit(1)
        alldics[onedic['host']] = onedic
    # when allics is ok, create the origin
    alldics_origin = copy.deepcopy(alldics)
    print("D... ALLDICS LOEADED",alldics)
    print("D... ALLDICS LOEADED",alldics_origin)



#----------------------------------------------------------- UPDATE
def update_table( okresults_dict , okvalues_dict={} ):
    global alldics, alldics_origin, tab_src
    tabsrc2 = tab_src.copy()


    neven = 0
    for i in alldics.keys():  # key is host, isnt it?
        #onedic = alldics[i]

        w = "autowhite"
        r = "autored"
        y = "autoyellow"
        g = "autogreen"
        abg = "autobgblack"

        # print( onedic )
        # print('D... updatetable - key',i)
        # print("D... u origins:",alldics_origin)
        # print("D... u origins I-hostname ok?:",alldics_origin[i]['hostname'] )


        # -------- ok_results dictionary :   fill it

        if 'ping' in okresults_dict.keys() and type(okresults_dict['ping']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['ping']:
                #print("OK")
                #alldics[i]['hostname'] = Color('{'+g+'}'+alldics_origin[i]['hostname']+'{/'+g+'}') # ONE COLOR ONLY
                alldics[i]['hostname'] = f"{fg.lightgreen}{alldics_origin[i]['hostname'].strip()}{fg.default}" # COLOR ONLY
                #print("...",alldics[i]['hostname'] )
            else:
                #alldics[i]['hostname'] = Color('{'+r+'}'+alldics_origin[i]['hostname']+'{/'+r+'}') # ONE COLOR ONLY
                alldics[i]['hostname'] = f"{fg.tomato}{alldics_origin[i]['hostname'].strip()}{fg.default}" # ONE COLOR ONLY
                #print("xxx",alldics[i]['hostname'] )


        if 'ssh' in okresults_dict.keys() and type(okresults_dict['ssh']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['ssh']:
                alldics[i]['user'] = Color('{'+g+'}'+alldics_origin[i]['user']+'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['user'] = Color('{'+r+'}'+alldics_origin[i]['user']+'{/'+r+'}') # ONE COLOR ONLY


        alldics[i]['inf'] = " "
        alldics[i]['ntp'] = " "
        alldics[i]['cam'] = " "
        #alldics[i]['vnc'] = " "
        # I spam 5678 no more
        #alldics[i]['mys'] = " "
        #alldics[i]['ser'] = " "
        #alldics[i]['bor'] = " "
        #alldics[i]['mgo'] = " "



        # INF
        if 'inf' in okresults_dict.keys() and type(okresults_dict['inf']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['inf']:
                # here I can perform another query?
                line = ""
                res = "OK "
                CMD = [ 'curl', '-G', '-m','2', f'http://{alldics_origin[i]["hostname"]}:8086/query?',
                        '--data-urlencode', 'q=show databases']
                #========================= this locks at some moment.
                try:
                    pipe = sp.Popen(CMD, stdin=sp.DEVNULL, stdout=sp.PIPE, stderr=sp.DEVNULL)
                    line = pipe.stdout.readline().decode("utf8").rstrip("\n")
                except Exception as ex:
                    print(ex)
                    line = ""

                if "i_am_" in line:
                    res = line.split("i_am_")[-1].split('"')[0]
                else:
                    res = "OK "
                alldics[i]['inf'] = Color('{'+g+'}'+res+'{/'+g+'}') # ONE COLOR ONLY
                #alldics[i]['inf'] = Color('{'+g+'}'+"OK "+'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['inf'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY


        # NTP
        if 'ntp' in okresults_dict.keys() and type(okresults_dict['ntp']) == list:
            myip = alldics_origin[i]['hostname']
            if myip in okresults_dict['ntp']: # if IP in []
                # print("D... okvalues",okvalues_dict)
                if myip in okvalues_dict.keys():
                    rx = okvalues_dict[myip]
                else:
                    rx = 0.0
                alldics[i]['ntp'] = Color('{'+g+'}'+ str(rx) +'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['ntp'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY


        # CAM
        if 'cam' in okresults_dict.keys() and type(okresults_dict['cam']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['cam']:
                alldics[i]['cam'] = Color('{'+g+'}'+"OK "+'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['cam'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY

        # VNC
        # if 'vnc' in okresults_dict.keys() and type(okresults_dict['vnc']) == list:
        #     if alldics_origin[i]['hostname'] in okresults_dict['vnc']:
        #         alldics[i]['vnc'] = Color('{'+g+'}'+"OK "+'{/'+g+'}') # ONE COLOR ONLY
        #     else:
        #         alldics[i]['vnc'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY

        # MYS
        if 'mys' in okresults_dict.keys() and type(okresults_dict['mys']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['mys']:
                alldics[i]['mys'] = Color('{'+g+'}'+"mys"+'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['mys'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY

        if 'ser' in okresults_dict.keys() and type(okresults_dict['ser']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['ser']:
                alldics[i]['ser'] = Color('{'+g+'}'+"OK "+'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['ser'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY

        if 'bor' in okresults_dict.keys() and type(okresults_dict['bor']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['bor']:
                alldics[i]['bor'] = Color('{'+g+'}'+"brg"+'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['bor'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY

        if 'mgo' in okresults_dict.keys() and type(okresults_dict['mgo']) == list:
            if alldics_origin[i]['hostname'] in okresults_dict['mgo']:
                alldics[i]['mgo'] = Color('{'+g+'}'+"brg"+'{/'+g+'}') # ONE COLOR ONLY
            else:
                alldics[i]['mgo'] = Color('{'+r+'}'+"---"+'{/'+r+'}') # ONE COLOR ONLY

        neven+=1
        if neven%3 == 0:
            oline =  [  #Color('{'+abg+'}'+alldics[i]['n'] + '{/'+abg+'}'),
                        f"{bg.darkslategray} {alldics[i]['n'].strip()} {bg.default}",
                        #Color('{'+abg+'}'+alldics[i]['host']    +'{/'+abg+'}'),
                        f"{bg.dimgray}{alldics[i]['host'].strip()}{bg.default}",
                        #Color('{'+abg+'}'+alldics[i]['user']    +'{/'+abg+'}'),
                        f"{bg.darkslategray}{alldics[i]['user'].strip()}{bg.default}",
                        #Color('{'+bg+'}'+alldics[i]['hostname']+'{/'+bg+'}'),
                        f"{bg.darkslategray}{alldics[i]['hostname'].strip()}{bg.default}",
                        #Color('{'+abg+'}'+alldics[i]['label']   +'{/'+abg+'}'),
                        f"{bg.darkslategray}{alldics[i]['label'].strip()}{bg.default}",
                        Color('{'+abg+'}'+alldics[i]['inf']     +'{/'+abg+'}'),
                        Color('{'+abg+'}'+alldics[i]['ntp']     +'{/'+abg+'}'),
                        Color('{'+abg+'}'+alldics[i]['cam']     +'{/'+abg+'}'),
                        #Color('{'+bg+'}'+alldics[i]['vnc']     +'{/'+bg+'}')
                        #Color('{'+bg+'}'+alldics[i]['mys']     +'{/'+bg+'}'),
                        #Color('{'+bg+'}'+alldics[i]['ser']     +'{/'+bg+'}'),
                        #Color('{'+bg+'}'+alldics[i]['bor']     +'{/'+bg+'}'),
                        #Color('{'+bg+'}'+alldics[i]['mgo']     +'{/'+bg+'}')
            ]
        else:
            oline =  [  f" {alldics[i]['n']} ",
                        alldics[i]['host'],
                        alldics[i]['user'],
                        alldics[i]['hostname'],
                        alldics[i]['label'],
                        alldics[i]['inf'],
                        alldics[i]['ntp'],
                        alldics[i]['cam'],
                        #alldics[i]['vnc']
                        #alldics[i]['mys'],
                        #alldics[i]['ser'],
                        #alldics[i]['bor'],
                        #alldics[i]['mgo']
            ]
            #oline = [ Color('{'+r+'}'+k+'{/'+r+'}') for k in oline ]
        tabsrc2.append( oline )
    #    print(onedic)
    return tabsrc2





def get_my_ip():
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    s.connect(("8.8.8.8", 80))
    MYIP=s.getsockname()[0]
    print("i... IP From 8.8.8.8:\t",s.getsockname()[0])
    s.close()
    return MYIP




def ping_port( IPT, port, ret ):
    """
    IPT = ip, port=port, ret=dict that is merged later
    if port == -1 ... ping
    if port is number ... conn/dicsonn
    ntp? -
    WHAT IS RET????
    """
    global DEBUG
    if DEBUG: print("D...                      "+IPT+":"+str(port) )
    if port == -1:
        command = 'ping -c 1 -w 2 '+ IPT
        res = sp.call(command.split(), stdin=sp.DEVNULL, stdout=sp.PIPE, stderr=sp.DEVNULL)
        if DEBUG: print("D.... .... ping....  ",IPT,res)
        ret[IPT] = res # impotrant
        return IPT, res

    if port == -2:
        command = 'ntpdate -q '+ IPT # ??????
        command = f"chronyc -h {IPT} tracking"
        #  Root delay .. max  error if maximum asymetry
        #  Root dispersion ,... min error estimation  travel_time * clockskew
        # command = 'ntpdig -dc '+ IPT  # last line ---
        #if DEBUG: print("D... ",command)
        #res =
        #ret[IPT] = 0
        ntp = 999
        try:
            pipe = sp.Popen(shlex.split(command), stdin=sp.DEVNULL, stdout=sp.PIPE, stderr=sp.DEVNULL)
            lines = pipe.stdout.readlines() #.decode("utf8").rstrip("\n")
            #print(lines)
            line = [x.decode("utf8").rstrip("\n") for x in lines if x.decode("utf8").find("Root delay")>=0 ][0]
            #print(line)
            line = line.strip().split(":")[-1].strip() # '0.004657954 seconds'
            #print("X1", line)
            ntp = line.split()[0].strip() # get seconds
            #print("X2", ntp)
            ntp = round(1000*abs(float( ntp )),1) # make rounded float
            #print("X3", ntp)
            #print("*x*", IPT, res)
            ret[IPT] = -1.0 * ntp # 0 #ntp # <0 is good NOT NICE...I NEED TO SEND <0
            res = 0 # good result
        except:
            ret[IPT] = 1 # BAD RESULT
            res = 1      # BAD
        # ------------------
        # try:
        #     pipe = sp.Popen(command.split(), stdin=sp.DEVNULL, stdout=sp.PIPE, stderr=sp.DEVNULL)
        #     line = pipe.stdout.readline().decode("utf8").rstrip("\n")
        #     if DEBUG: print("D1... ... line = ",line)
        #     if len(line.split())>4:
        #         res = 0
        #         ntp = round(1000*abs(float( line.split()[3] )),1)
        #     else:
        #         if DEBUG: print("D... ... split failed, leaving")
        #         res = 1
        #         ntp = -1
        #     if DEBUG: print("D.... .... ntpsec-ntpdate....  ",IPT,res, ntp)
        #     ret[IPT] = -1.0*ntp # important? - 0/1 .... or <0with ntp
        # except:
        #     ret[IPT] = 1
        #     res = 1
        #print("---- return", IPT, res, ret[IPT], "----")
        return IPT, res

    sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    sock.settimeout(2)
    try:
        result = sock.connect_ex((IPT, port))
    except socket.error:
        print(f"!... maybe NO DNS for {IPT}:{port}")
        result = -1

#    if result == 0:
#        print(" Open")
#    else:
#        print(" ----")
    sock.close()
    ret[IPT]=result # important
    return IPT,result






# 8086 influx
def discover_port(iplist, port='ssh'):
    """
    checks ports or PING
    """
    ret={}
    t=[]
    okresults = []
    okvalues = {}

    if port=='ssh':
        portn = 22
    elif port == 'ntp':
        portn = -2
    elif port == 'cam':
        portn = 8000
    elif port == 'inf':
        portn = 8086
    #elif port == 'vnc':
    #    portn = 5900
    #elif port == 'mys':
    #    portn = 5678
    elif port == 'ping':
        portn = -1
    #elif port == 'bor':
    #    portn = 2222
    #elif port == 'mgo':
    #    portn = 27017
    #elif port == 'ser':
    #    portn = 8099
#   # elif port == 'ser':
#   #     portn = 9010
    else:
        return


    if DEBUG: print("D... looking for port",port, portn)

    #for i in range(1,254):
    #for i in range(60,63):
    for i in iplist:
        #IPT=IP3+"."+str(i)
        IPT = i
        t.append( Thread(target=ping_port,args=(IPT,portn,ret)) )
        t[-1].start()
    for i in t:
        i.join()
    #print("\n",ret) #================== ret contains
    # print()


    for i,v in ret.items():
        #print(i,v)
        if v<=0:
            okresults.append( i ) # append the IP
        if v==0:
            okvalues[i]=0 # append the IP
        else: # <0 can be NTPQ
            okvalues[i]= -v  # append the IP


    # print("i... ok==", sorted(okresults), sorted(okvalues) )

    # with open( os.path.expanduser("~/.myservice_discover8086"),"w") as f:
    #      for i in okresults:
    #          print("D... writing to dicover",i)
    #          f.write( i +"\n")
    #          write_influx(i,MYIP) #
    return okresults, okvalues






def flat_scan(port = 22):
    """
    scan all range if local IP adresses
    """
    MYIP = get_my_ip()
    IP3=".".join( MYIP.split(".")[:3] )
    print("i... Scan ",IP3 +".*")
    iplist = list(range(1,254))

    discover_port(iplist,  port)









def main():#  param="ok", debug = False):
    global DEBUG, real_width, pause_main
    # toggles.disable_all_colors()

    # --------------------- replacement ------------------------------
    param=f"ok"
    debug = False

    # ----------wacky way
    if len(sys.argv) < 2:
        print("USAGE:  <par> [par2 ...]")
    else:
        n = 1
        for i in sys.argv[1:]:
            if n == 1:
                param = i
            elif i == "-d":
                debug = True

    print(param, debug)
    if param.find("~/.ssh/") >= 0:
        param = param.split("~/.ssh/")[-1]
    if param.find(os.path.expanduser("~/.ssh/")) >= 0:
        param = param.split(os.path.expanduser("~/.ssh/"))[-1]

    DEBUG = debug
    print(param, debug)
    #sys.exit(0)

    load_resources(param)
    # print( list(alldics.keys() ) )
    #sys.exit(0)

    #start the Keyboard thread
    kthread = KeyboardThread(my_callback)

    i = 0 # SCANS ALL AT ONCE
    #i = 1 # SCANS later
    okresults_dict = {}
    okvalues_dict = {}
    while True:
        # --------- here we discover different ports at different times -----------
        #print("D... WHILE",alldics_origin)
        i+=1
        DT = 120 # seconds
        # then every 3 seconds one port is scanned over all IP
        if (i%(10)) == 1:
            iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
            okresults_dict['ping'],_ = discover_port(iplist, 'ping')
        if (i%(DT)) == 6:
            iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
            okresults_dict['ssh'],_ = discover_port(iplist, 'ssh')
        # if (i%(DT)) == 27:
        #     iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
        #     okresults_dict['vnc'],_ = discover_port(iplist, 'vnc')
        if (i%(DT)) == 9:
            iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
            okresults_dict['cam'],_ = discover_port(iplist, 'cam')
        if (i%(DT)) == 12:
            iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
            okresults_dict['inf'],_ = discover_port(iplist, 'inf')
        #if (i%(DT)) == 15:
        #    iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
        #    okresults_dict['mys'],_ = discover_port(iplist, 'mys')
        # if (i%(DT)) == 18:
        #     iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
        #     okresults_dict['bor'],_ = discover_port(iplist, 'bor')
        # if (i%(DT)) == 21:
        #     iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
        #     okresults_dict['mgo'],_ = discover_port(iplist, 'mgo')
        # if (i%(DT)) == 24:
        #     iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
        #     okresults_dict['ser'],_ = discover_port(iplist, 'ser')

        # this is special... return values of ntpq
        if (i%(DT)) == 3:
            iplist = [ alldics_origin[i]['hostname'] for i in list(alldics.keys()) ]
            #print("D... ", iplist)
            res = discover_port(iplist, 'ntp')
            #print("D...", res)
            okresults_dict['ntp'],okvalues_dict = res


        print(f" ... done {i} {i%DT}",end="")
        if (i%DT> 27) and (i%DT<120):
            print(f" ... ping is <10s old; other {i%DT-27}+ sec")
        else:
            print(" ")
        time.sleep(0.5)

        tabsrc2 = update_table( okresults_dict, okvalues_dict )


        table = SingleTable(tabsrc2)
        table.padding_left = 1
        table.padding_right = 1

        if not debug:
            print( t.clear(),end="" )

        if not table.ok:
            table.padding_left=0
        if not table.ok:
            table.padding_right=0
        while not table.ok:
            j=0
            for k in tabsrc2:
                tabsrc2[j] = k[:-1]
                j+=1
            table = SingleTable(tabsrc2)
            table.padding_left = 0
            table.padding_right = 0

        # ========================================================================
        # ========================================================================
        # ======= THIS IS PRINT TABLE MOMENT   ===================================
        # ========================================================================
        # ========================================================================
        print(table.table )

        print(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")," ...(c)am (m)ys (s)sh (S)sh (p)ingy (v)nc (R)sync (h)elp")
        time.sleep(1)
        while pause_main:
            time.sleep(0.5)


if __name__ == "__main__":
    main()
